/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.similarity.data;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.openscience.cdk.exception.CDKException;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.qsar.DescriptorSpecification;
import org.openscience.cdk.qsar.DescriptorValue;
import org.openscience.cdk.qsar.IMolecularDescriptor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ambit.data.descriptors.Descriptor;
import ambit.data.literature.LiteratureEntry;
import ambit.processors.Builder3DProcessor;
import chemCast.events.PropertyEnabledChangeEvent;

public class ChemCastDescriptor extends Descriptor implements IChemCastObject, IMolecularDescriptor {
    protected static String types[]={"2D","3D"};
	protected PropertyChangeSupport propertyChangeSupport;
	public final static String p_descriptor = "descriptor";
	public final static String p_note = "note";
    public final static String p_type = "type";
    public final static String p_weight = "weight";
	protected String descriptorClass = null;
	protected String tag = null;
	protected boolean enabled = false;
	protected IMolecularDescriptor calculator;
    protected boolean required3D = false;
    protected static Builder3DProcessor builder3D = null;
    protected double weight = 0;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 3676566999297559204L;

	public ChemCastDescriptor(String name, LiteratureEntry reference) {
		super(name, reference);
        propertyChangeSupport = new PropertyChangeSupport(this);
        setEnabled(true);
	}

	public ChemCastDescriptor(String name, int type, LiteratureEntry reference) {
		super(name, type, reference);
        propertyChangeSupport = new PropertyChangeSupport(this);
        setEnabled(true);
	}

	public ChemCastDescriptor(String name, int type, int dtype,
			LiteratureEntry reference) {
		super(name, type, dtype, reference);
        propertyChangeSupport = new PropertyChangeSupport(this);
        setEnabled(true);
	}

	public PropertyChangeSupport getPropertyChangeSupport() {
		return propertyChangeSupport;
	}

	public void setPropertyChangeSupport(
			PropertyChangeSupport propertyChangeSupport) {
		this.propertyChangeSupport = propertyChangeSupport;

	}

	public Element getXMLElement(Document document) {
        Element e = document.createElement(p_descriptor);
        e.setAttribute(p_name, getName());
       	e.setAttribute(p_note,getRemark().toString());
        e.setAttribute(p_class,getDescriptorClass());
        e.setAttribute(p_tag,getTag());
        e.setAttribute(p_weight,Double.toString(getWeight()));
        if (isRequired3D()) e.setAttribute(p_type,types[1]);
        else e.setAttribute(p_type,types[0]);
        return e;
	}

	public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_descriptor)) {
            setName(element.getAttribute(IChemCastObject.p_name));
            setDescriptorClass(element.getAttribute(IChemCastObject.p_class));
            setRemark(element.getAttribute(p_note));
            try {
            	setWeight(Double.parseDouble(element.getAttribute(p_weight)));
            } catch (Exception x) {
            	setWeight(0);
            }
            String type = element.getAttribute(p_type);
            setRequired3D((types[1].equals(type)));
        } else throw new Exception("Not an element of type "+p_descriptor);
	}

	public void firePropertyChange(String propertyName, Object oldValue,
			Object newValue) {
		propertyChangeSupport.firePropertyChange(propertyName,oldValue,newValue);

	}

	public void firePropertyChange(PropertyChangeEvent event) {
		propertyChangeSupport.firePropertyChange(event);

	}

	public void addPropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(propertyName, listener);

	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);

	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(listener);

	}

	public void removePropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		propertyChangeSupport.removePropertyChangeListener(propertyName, listener);
	}

	public boolean isEnabled() {
		return enabled;
	}

	public void setEnabled(boolean enabled) {
    	boolean oldValue = this.enabled;
    	this.enabled = enabled;
        firePropertyChange(
                new PropertyEnabledChangeEvent(this,"enable",oldValue,enabled));
	}

	public String getDescriptorClass() {
		return descriptorClass;
	}

	public void setDescriptorClass(String descriptorClass) {
		this.descriptorClass = descriptorClass;
		setEnabled((descriptorClass!=null) && !descriptorClass.equals(""));
        firePropertyChange(
                new PropertyChangeEvent(this,p_class,null,descriptorClass));		
	}
	public String[] getParameterNames() {
		return getCalculator().getParameterNames();
	}
	public Object[] getParameters() {
		return getCalculator().getParameters();
	}
	public Object getParameterType(String arg0) {
		return getCalculator().getParameterType(arg0);
	}
	public DescriptorSpecification getSpecification() {
		return getCalculator().getSpecification();
	}
	public void setParameters(Object[] arg0) throws CDKException {
		getCalculator().setParameters(arg0);
		
	}
	public DescriptorValue calculate(IAtomContainer a) throws CDKException {
        /*
        if (isRequired3D()) {
            boolean coordinates2D = true;
            boolean coordinates3d = true;
            
            for (int i=0; i < a.getAtomCount();i++) {
                if (a.getAtomAt(i).getPoint3d() == null) coordinates3d = false;
                if (a.getAtomAt(i).getPoint2d() == null) coordinates2D = false;
            }
            if (!coordinates3d) {
                if (builder3D == null) builder3D = new Builder3DProcessor(
                        new FastTemplateHandler3D());
                try {
                    builder3D.process(a);
                } catch (AmbitException x) {
                    throw new CDKException(x.getMessage());
                    
                }

            }
        }
        */
		return getCalculator().calculate(a);
	}

	public IMolecularDescriptor getCalculator() {
		if (calculator == null) 
			try {
				Class c = Class.forName(descriptorClass);
				calculator = (IMolecularDescriptor)c.newInstance();
				setEnabled(true);
			} catch (Exception x) {
				System.err.println(getName() + "\tDescriptor classname\t"+ descriptorClass);
				//x.printStackTrace();
				calculator = null;
				setEnabled(false);
			}		
		
		return calculator;
	}

	@Override
	public String toString() {
		return getName();
	}

	public String getTag() {
		if (tag==null) return getName();
		else return tag;
	}

	public void setTag(String tag) {
		this.tag = tag;
        firePropertyChange(
                new PropertyChangeEvent(this,p_tag,null,tag));
	}

    public synchronized boolean isRequired3D() {
        return required3D;
    }

    public synchronized void setRequired3D(boolean required3D) {
        this.required3D = required3D;
    }

	public double getWeight() {
		return weight;
	}

	public void setWeight(double weight) {
		this.weight = weight;
	}
}


