/* NNAtomEnvironmentSimilaritySubsets.java
 * Author: Nina Jeliazkova
 * Date: Feb 4, 2007 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.util.BitSet;

import org.openscience.cdk.interfaces.IAtomContainer;

import ambit.data.descriptors.AtomEnvironmentList;
import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.AtomEnvironmentGenerator;
import ambit.similarity.AtomEnvironmentsDistance;
import ambit.similarity.IDistanceFunction;

public class NNAtomEnvironmentSimilaritySubsets extends
        NearestNeighborsSimilaritySubsets<AtomEnvironmentList> {

    /**
     * 
     */
    private static final long serialVersionUID = 6487290843144033607L;
    public static String p_use_hydrogens = "hydrogens";
    protected AtomEnvironmentGenerator generator;

    public NNAtomEnvironmentSimilaritySubsets() {
        this(10);
    }    
    public NNAtomEnvironmentSimilaritySubsets(int knn) {
        this(new AtomEnvironmentsDistance(), knn);
    }    
    public NNAtomEnvironmentSimilaritySubsets(IDistanceFunction<AtomEnvironmentList> distanceFunction, int knn) {
        super(distanceFunction, knn);
        generator = new AtomEnvironmentGenerator();
        generator.setUseHydrogens(true);
        
        getNearestNeighborsSearch().setLowestDistanceIsClosest(false);

    }

    @Override
    protected AtomEnvironmentList calculateComparableProperty(Object object)
            throws AmbitException {
        IAtomContainer ac = ((IAtomContainer)object);
        Object ae = ac.getProperty(AmbitCONSTANTS.AtomEnvironment);
        if ((ae != null) && (!"".equals(ae)) && (ae instanceof AtomEnvironmentList)) 
            return (AtomEnvironmentList) ae;
        generator.process(ac);
        ae = ac.getProperty(AmbitCONSTANTS.AtomEnvironment);
        if (ae instanceof AtomEnvironmentList) {
            if (((AtomEnvironmentList) ae).size() == 0) return null;
            else return (AtomEnvironmentList) ae; 
        }
        else return null;
    }
    @Override
    protected AtomEnvironmentList readComparableProperty(IAtomContainersList list, int index) throws AmbitException {
        Object ae = list.getProperty(index, AmbitCONSTANTS.AtomEnvironment);
        if ((ae != null) && (!"".equals(ae)) && (ae instanceof AtomEnvironmentList)) 
            return (AtomEnvironmentList) ae;        
        else return null;
    }
    @Override
    protected void writeComparableProperty(IAtomContainersList list, int index, AtomEnvironmentList property) throws AmbitException {
        list.setProperty(index, AmbitCONSTANTS.AtomEnvironment, property);
        
    }
    public void setParameter(Object parameter, Object value) {
        super.setParameter(parameter, value);
        if (parameter.equals(p_use_hydrogens)) 
        try {
            boolean h = Boolean.parseBoolean(value.toString());
            generator.setUseHydrogens(h);
        } catch (Exception x) {
            generator.setUseHydrogens(false);
            logger.error(x);
        }
 

    }
    public Object getParameter(Object parameter) {
        Object p = super.getParameter(parameter);
        if (p!=null) return p;
        else if (p_use_hydrogens.equals(parameter)) return new Boolean(generator.isUseHydrogens());
        else return null;
    }
	@Override
	protected int getNumberOfAttributes() {
		// TODO Auto-generated method stub
		return 0;
	}
	@Override
	protected Object readAttribute(AtomEnvironmentList attributes, int attribute_index) {
		return null;
	}
	@Override
	protected double readMaxAttribute(int attribute_index) {
		return 0;
	}
	@Override
	protected double readMinAttribute(int attribute_index) {
		return 1;
	}
	@Override
	protected AtomEnvironmentList getSelectedAttributes(AtomEnvironmentList attributes) {
		return attributes;
	}

}
