/* ChemCASTModuleTest.java
 * Author: Nina Jeliazkova
 * Date: Sep 30, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test.data;

import java.io.File;
import java.util.List;

import javax.swing.JOptionPane;

import junit.framework.TestCase;
import ambit.data.molecule.IAtomContainersList;
import ambit.misc.AmbitCONSTANTS;
import ambit.similarity.ISimilarityProcessor;
import chemCast.application.ChemCASTModule;
import chemCast.data.IObjectFilter;
import chemCast.similarity.data.ChemCastDescriptor;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Endpoints;
import chemCast.similarity.data.SimilarityDef;
import chemCast.similarity.data.SimilarityDefs;
import chemCast.similarity.data.SimilarityModel;
import chemCast.similarity.data.Subsets;
import chemCast.ui.ListPanel;
import chemCast.ui.actions.SimilarityCalculationAction;
import chemCast.weka.AbstractWekaModel;
import chemCast.weka.WekaSimilarityModel;

public class ChemCASTModuleTest extends TestCase {
    /*
    public void test() {
        ChemCASTModule m = new ChemCASTModule();
        PropertyChangeSupport ps = new PropertyChangeSupport(m);
        ps.addPropertyChangeListener(m);
        try {
            File file = new File("data/LLNA.sdf");
            MoleculesFile mf = new MoleculesFile(file, DefaultChemObjectBuilder.getInstance());

            ps.firePropertyChange(new TrainingDatasetChangeEvent(
                    this,
                    TrainingDatasetChangeEvent.op_add,
                    null,
                    mf
                    ));
            assertEquals(1,m.getTrainingSet().size());
            assertEquals(209,m.getTrainingSet().get(0).getAtomContainerCount());
            FilteredMoleculesFile m1 = new FilteredMoleculesFile(mf,new PropertyFilter("Vehicle1","AOO"));
            ps.firePropertyChange(new TrainingDatasetChangeEvent(
                    this,
                    TrainingDatasetChangeEvent.op_add,
                    null,
                    mf
                    ));
            assertEquals(2,m.getTrainingSet().size());
            ps.firePropertyChange(new TrainingDatasetChangeEvent(
                    this,
                    TrainingDatasetChangeEvent.op_clear,
                    null,
                    mf
                    ));
            assertEquals(0,m.getTrainingSet().size());
        } catch (Exception x) {
            fail();
        }
    }
    */
	public void testDescriptors() {
		ChemCastDescriptorsList d = ChemCASTModule.readDescriptors();
		for (int i=0; i < d.getList().size();i++) {
			ChemCastDescriptor c = d.get(i);
			System.out.print(c.getName());
			System.out.print("\t");
			System.out.print(c.getRemark());System.out.print("\t");
			System.out.print(c.getDescriptorClass());System.out.print("\n");
			assertNotNull(c.getName());
			assertNotNull(c.getRemark());
			assertNotNull(c.getDescriptorClass());
			assertEquals(c.getCalculator()!=null,c.isEnabled());
		}	
		ListPanel p = new ListPanel("Descriptors",d);
		JOptionPane.showMessageDialog(null,p);
	}
	
	public void testAll() {
		
		ChemCASTModule m = new ChemCASTModule();
		SimilarityCalculationAction action = new SimilarityCalculationAction(m,null,ChemCASTModule.DATA_TRAINING,ChemCASTModule.DATA_TRAINING);
		Endpoints endpoints = m.readEndpoints();
		for (int e=0; e< endpoints.size(); e++) {
			System.out.println(endpoints.get(e));
			System.out.println("");
			DatasetDef dataset = endpoints.get(e).getTrainingData();
			try {
				IAtomContainersList mf = dataset.createData(null,(IObjectFilter)null);
				assertTrue(mf.getAtomContainerCount()>0);
			} catch (Exception x) {

				File file = new File(dataset.getDatasetValue());
				if (file.exists()) {
					x.printStackTrace();
					fail();
				}	
			}
			try {
				List<Subsets> list_of_subsets = endpoints.get(e).getSubsets();
				for (int l=0; l< list_of_subsets.size();l++) {
					m.getDataset(ChemCASTModule.DATA_TRAINING).createSubsets(dataset,list_of_subsets.get(l));	
					assertTrue(m.getDataset(ChemCASTModule.DATA_TRAINING).getList().size()>0);
					
					/*
					for (int d=0; d < m.getDataset(ChemCASTModule.DATA_TRAINING).getList().size(); d++) {
						IAtomContainersList data = m.getDataset(ChemCASTModule.DATA_TRAINING).get(d).getData();
						assertNotNull(data);
					}
					*/
					
					SimilarityDefs sim_methods = endpoints.get(e).getSimilarityMethods();
					assertTrue(sim_methods.getList().size()>0);
					
					System.out.println(list_of_subsets.get(l));
					System.out.println("");
					for (int s=0; s < sim_methods.getList().size();s++) {
						SimilarityModel similarity = new SimilarityModel(sim_methods.get(s));
						
						similarity.setSubsetIndices(new int[] {0});
						m.getDataset(ChemCASTModule.DATA_TRAINING).setSimilarity(similarity);
						SimilarityModel newSimilarity = m.getDataset(ChemCASTModule.DATA_TRAINING).getSelectedProcessor();
						System.out.print(newSimilarity);
						ISimilarityProcessor processor = newSimilarity.getProcessor();
						assertNotNull(processor);
						
						if (processor instanceof AbstractWekaModel) {
							Object hasFingerprints = processor.getParameter("Fingerprints");
							if ((hasFingerprints==null) || hasFingerprints.equals("False")) {
								ChemCastDescriptorsList descriptors = newSimilarity.getDescriptors();
								assertNotNull(descriptors);
								assertTrue(descriptors.getList().size()>0);
							}
						}
						System.out.print("\tOK");
						/*
						try {
							action.process(similarity, m.getDataset(ChemCASTModule.DATA_TRAINING));
							System.out.println("\tOK");
						} catch (Exception x) {
							System.out.println(x.getMessage());
						}
						*/
						System.out.println();
					}
				}
				
			} catch (Exception x) {
				x.printStackTrace();
				fail();
			}
				
		}
	}
}
