/* SubsetTest.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test.data;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import chemCast.similarity.data.ChemCastObjectFactory;
import chemCast.similarity.data.Dataset;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

/**
 * Test for {@link Subset}
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Sep 23, 2006
 */
public class SubsetTest extends TestCase {

    /**
     * @param arg0
     */
    public SubsetTest(String arg0) {
        super(arg0);
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
    public void testCreate() {
    	try {
    	Hashtable p = new java.util.Hashtable();
    	p.put("label","MyLabel");
        Subset s = new Subset("MyName",p,Color.blue);
        assertEquals("MyLabel",s.getProperty("label"));
        assertEquals(Color.blue,s.getColor());
        
        s.setPropertyChangeSupport(new PropertyChangeSupport(s));
        s.addPropertyChangeListener("name", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                assertEquals("name",evt.getPropertyName());
                assertEquals("MyName",evt.getOldValue());
                assertEquals("newname",evt.getNewValue());
                assertTrue(evt.getSource() instanceof Subset);
                ((Subset) evt.getSource()).setProperty("label","newlabel");
            }
        });
        s.addPropertyChangeListener("label", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                assertEquals("label",evt.getPropertyName());
                assertEquals("MyLabel",evt.getOldValue());
                assertEquals("newlabel",evt.getNewValue());
                assertTrue(evt.getSource() instanceof Subset);
                ((Subset) evt.getSource()).setColor(Color.red);
            }
        });        
        s.addPropertyChangeListener("color", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                assertEquals("color",evt.getPropertyName());
                assertEquals(Color.blue,evt.getOldValue());
                assertEquals(Color.red,evt.getNewValue());
            }
        });   
        s.setName("newname");
        assertEquals("newlabel",s.getProperty("label"));
        assertEquals(Color.red,s.getColor());
    	} catch (Exception x) {
    		fail();
    	}
    }
    public void testToXML() {

        try {
          	Hashtable p = new java.util.Hashtable();
        	p.put("label","MyLabel");
        	p.put("tag","Property");
            Subset s = new Subset("MyName",p,Color.blue);
            
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            Element e = s.getXMLElement(doc);
            assertEquals("MyName", e.getAttribute(Subset.p_name));
            assertEquals("MyLabel", e.getAttribute("label"));
            assertEquals("Property", e.getAttribute("tag"));
            assertEquals(Color.blue,Dataset.Hex2Color(e.getAttribute(Subset.p_color)));
            doc.appendChild(e);
            ChemCastObjectFactory.writeXmlFile(doc, "test.xml");
            Subset s1 = new Subset();
            s1.setXMLElement(e);
            assertEquals(s,s1);
        } catch (Exception e) {
            e.printStackTrace();
            fail();
        }


    }
    public void testlistToXML() {
    	
       	
        try {
        	Hashtable p1 = new java.util.Hashtable();
        	p1.put("label","MyLabel1");
        	p1.put("tag","Property");
            Subset s = new Subset("MyName1",p1,Color.blue);
            Hashtable p2 = new java.util.Hashtable();
        	p2.put("label","MyLabel2");
        	p2.put("tag","Property");
            Subset s1 = new Subset("MyName2",p2,Color.red);
            
            ArrayList<IChemCastObject> l = new ArrayList<IChemCastObject>();
            l.add(s); l.add(s1);
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            Element e = ChemCastObjectFactory.getXMLElement(l, doc, "groups");
            doc.appendChild(e);
            assertEquals(2,e.getChildNodes().getLength());
            ChemCastObjectFactory.writeXmlFile(doc, "test.xml");
            
            NodeList nodes = doc.getElementsByTagName(Subsets.p_groups);
            Subsets sets = new Subsets((Element)nodes.item(0));
            assertEquals(2,sets.size());
        } catch (Exception x) {
            fail();
        }
    }
    public void testColor() {
        Color c = Color.blue;
        try {
        assertEquals(c,Dataset.Hex2Color(Dataset.Color2Hex(c)));
        } catch (Exception x) {
            x.printStackTrace();
            fail();
        }
    }
}
