/* ListPanel.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import chemCast.similarity.data.IChemCastList;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.Subsets;

/**
 * Displays {@link IChemCastList}.
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Sep 23, 2006
 */
public class ListPanel extends JPanel implements PropertyChangeListener{
    protected JTable table;
    protected TitledBorder border;
    protected String caption;
    protected JToolBar bar;
    protected ChemCastListTableModel model;
    /**
     * 
     */
    private static final long serialVersionUID = 5554385761977043924L;

    /**
     * 
     */
    public ListPanel(String caption, IChemCastList collection) {
        super();
        ActionMap actions = new ActionMap();
        actions.put("select", new SelectAction(true,collection));
        actions.put("unselect", new SelectAction(false,collection));
        addWidgets(caption,collection,actions);
    }
    public ListPanel(String caption, IChemCastList collection,ActionMap actions) {
        super();
        addWidgets(caption,collection,actions);
    }
    public void addWidgets(String caption,IChemCastList collection,ActionMap actions) {
        this.caption = caption;
        setMinimumSize(new Dimension(200,200));
        setPreferredSize(new Dimension(300,300));
        setLayout(new BorderLayout());
        model = new ChemCastListTableModel(collection);
        table = new JTable(model,createColumnModel(collection.getList()));
        table.addMouseListener(new MouseAdapter() {
        	@Override
        	public void mouseClicked(MouseEvent e) {
        		super.mouseClicked(e);
            	clickRow(table,table.rowAtPoint(e.getPoint()));
        	}
        });
        table.setRowSelectionAllowed(true);
        table.setColumnSelectionAllowed(false);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.getTableHeader().setReorderingAllowed(false);
        table.setShowHorizontalLines(false);
        table.setShowVerticalLines(false);
        JScrollPane scrollPane = new JScrollPane(table);
        add(scrollPane,BorderLayout.CENTER);
        scrollPane.setMinimumSize(new Dimension(200,200));
        scrollPane.setPreferredSize(new Dimension(200,200));
        border = BorderFactory.createTitledBorder(caption);
        setBorder(border);
        bar = new JToolBar();
        add(bar,BorderLayout.SOUTH);
        //setActionMap(actions);
        setActions(actions);
    }
    public void setActions(ActionMap actions) {
        if (actions ==null) return;
        bar.removeAll();
        Object[] keys = actions.keys();
        for (int i=0; i < keys.length;i++) {
            bar.add(new JButton(actions.get(keys[i])));
        }        
        
    }
    public void setList(String caption, IChemCastList list) {
        ((ChemCastListTableModel)table.getModel()).setList(list);
        setCaption(caption);
    }
    public static TableColumnModel createColumnModel(List<? extends IChemCastObject> list) {
        TableColumnModel m = new DefaultTableColumnModel();
        m.addColumn(new TableColumn(0,32));
        m.addColumn(new TableColumn(1,200));
        return m;
    }

    public void addListSelectionListener(ListSelectionListener listener) {
        table.getSelectionModel().addListSelectionListener(listener);
    }
    public void removeListSelectionListener(ListSelectionListener listener) {
        table.getSelectionModel().removeListSelectionListener(listener);
    }
    public synchronized JTable getTable() {
        return table;
    }
    /*
    public synchronized int getSelectedIndex() {
        return selectedIndex;
    }
    public synchronized void setSelectedIndex(int selectedIndex) {
        this.selectedIndex = selectedIndex;
    }
    */
    public synchronized String getCaption() {
        return caption;
    }
    public synchronized void setCaption(String caption) {
        this.caption = caption;
        border.setTitle(caption);
    }
    public void propertyChange(PropertyChangeEvent evt) {
    	if (evt.getPropertyName().equals(Subsets.p_groups)) 
            if (evt.getNewValue()!=null)
    		setList(evt.getNewValue().toString(),(Subsets) evt.getNewValue());
    	
    }
    @Override
    public void setEnabled(boolean enabled) {
    	super.setEnabled(enabled);
    	model.setEnabled(enabled);
    }
    public void clickRow(JTable table , int row) {
    	
    }
}

class SelectAction extends AbstractAction {
	boolean selected = true;
	protected IChemCastList list;
	public SelectAction(boolean selected, IChemCastList list) {
		super(selected?"Select All":"Unselect all");
		this.selected = selected;
		this.list = list;
	}
	public void actionPerformed(ActionEvent e) {
		list.selectAll(selected);
		
	}
}