/* MoleculesTable.java
 * Author: Nina Jeliazkova
 * Date: Sep 30, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileOutputStream;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.JToolBar.Separator;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import nu.xom.Element;

import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.IMolecule;
import org.openscience.cdk.io.IChemObjectWriter;
import org.openscience.cdk.smiles.SmilesParser;
import org.openscience.cdk.templates.MoleculeFactory;

import uk.ac.cam.ch.wwmm.opsin.NameToStructure;
import ambit.data.molecule.Compound;
import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.IMoleculesIterator;
import ambit.io.FileInputState;
import ambit.io.FileOutputState;
import ambit.io.MyIOUtilities;
import ambit.ui.UITools;
import ambit.ui.actions.process.MoleculeEditAction;
import ambit.ui.data.ImageCellRenderer;
import ambit.ui.data.MoleculeGridCellRenderer;
import chemCast.application.StatusBar;
import chemCast.data.RandomAccessSubsetTableModel;
import chemCast.data.SelectedItem;
import chemCast.events.ViewModeChangeEvent;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.Subset;

/**
 * Listens for:
 * <ul>
 * <li>{@link ViewModeChangeEvent} to change {@link #isSpreadsheet()}
 * <li>{@link PropertyChangeEvent} with propertyname =
 * {@link IChemCastObject.p_property} to add/remove properties to be displayed
 * <li>{@link PropertyChangeEvent} with {@link SelectedItem} new value to
 * select the item and/or load {@link SelectedItem#getList()}
 * </ul>
 * TODO [S-]C1=NC(C(C(C)CCC)(CC)C(N1)=O)=O.[Na+]  - doesn't display
 * @author Nina Jeliazkova
 * 
 */
public class MoleculesTable extends JPanel implements PropertyChangeListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3304388795456139707L;

	protected JTable table;

	protected JLabel label;

	protected boolean spreadsheet = true;

	protected final Dimension cellSize;

	public MoleculesTable(RandomAccessSubsetTableModel model, Dimension cellSize) {
		super(new BorderLayout());
		this.cellSize = cellSize;
		table = addWidgets(model);
		label = new JLabel(model.toString());
		JScrollPane p = new JScrollPane(table);
		p.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		table.setPreferredScrollableViewportSize(new Dimension(600, 200));
        table.setDefaultRenderer(Hashtable.class, new PropertiesCellRenderer());
        table.setDefaultEditor(Hashtable.class, new PropertiesCellRenderer());
		add(p, BorderLayout.CENTER);
		add(label, BorderLayout.NORTH);
		add(addControls(), BorderLayout.SOUTH);
		setMinimumSize(new Dimension(200, 200));
		setSpreadsheet(false);
	}

	protected JComponent addControls() {
		JToolBar p = new JToolBar();
		p.setFloatable(false);
        p.add(new MoleculesTableAction("Save",UITools.createImageIcon("ambit/ui/images/save_16.png"),"Save to file") {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        });                  
        p.add(new MoleculesTableAction("Info",UITools.createImageIcon("ambit/ui/images/dataset.png"),"Dataset information") {
            public void actionPerformed(ActionEvent e) {
                info();
            }
        });                     
        p.add(new Separator());
		p.add(new MoleculesTableAction("Go to record",UITools.createImageIcon("ambit/ui/images/enter.png"),"Go to record number") {
			public void actionPerformed(ActionEvent e) {
				gotoRecord();
			}
		});
		p.add(new MoleculesTableAction("Search",UITools.createImageIcon("ambit/ui/images/search.png"),"Search by CAS RN") {
			public void actionPerformed(ActionEvent e) {
				lookup(CDKConstants.CASRN);
			}
		});
		/*
		p.add(new MoleculesTableAction("Find",UITools.createImageIcon("ambit/ui/images/search.png"),"Find similar compounds") {
			public void actionPerformed(ActionEvent e) {
				try {
					findSimilar(CDKConstants.CASRN);
				} catch (Exception x) {
					x.printStackTrace();
				}
			}
		});
		*/		
		p.add(new Separator());
		p.add(new MoleculesTableAction("Edit",UITools.createImageIcon("ambit/ui/images/draw_16.png"),"Edit current structure") {
			public void actionPerformed(ActionEvent e) {
				
				editAtomContainter();
				//editProperties();
				
			}
		});			
		p.add(new MoleculesTableAction("Append",UITools.createImageIcon("ambit/ui/images/add.gif"),"Append a new record") {
			public void actionPerformed(ActionEvent e) {
				addAtomContainter();
			}
		});
		p.add(new MoleculesTableAction("Delete",
                UITools.createImageIcon("ambit/ui/images/delete.gif"),"Delete current record") {
			public void actionPerformed(ActionEvent e) {
				deleteAtomContainter();
			}
		});
	
		p.add(new Separator());
		p.add(new MoleculesTableAction("First",UITools.createImageIcon("ambit/ui/images/nav_first.gif"),
        "Go to the first record") {
	public void actionPerformed(ActionEvent e) {
		first();
	}
		});
		p.add(new MoleculesTableAction("Prev",UITools.createImageIcon("ambit/ui/images/nav_prev.gif"),
		"Go to the previous record") {
		    public void actionPerformed(ActionEvent e) {
		        prev();
		    }
		});
		p.add(new MoleculesTableAction("Next",UITools.createImageIcon("ambit/ui/images/nav_next.gif"),
		"Go to the next record") {
		    public void actionPerformed(ActionEvent e) {
		        next();
		    }
		});
		        
		p.add(new MoleculesTableAction("Last",UITools.createImageIcon("ambit/ui/images/nav_last.gif"),"Go to the last record") {
			public void actionPerformed(ActionEvent e) {
				last();
			}
		});		


		 
		return p;

	}

	protected JTable addWidgets(RandomAccessSubsetTableModel model) {

		JTable table = new JTable(model) {
			public void createDefaultColumnsFromModel() {
				TableModel m = getModel();
				if (m != null) {
					// Remove any current columns
					TableColumnModel cm = getColumnModel();
					while (cm.getColumnCount() > 0) {
						cm.removeColumn(cm.getColumn(0));
					}
					// Create new columns from the data model info
					final int columnSize[] = { 32, 64, 100, 48, 48 };
					for (int i = 0; i < m.getColumnCount(); i++) {
						TableColumn newColumn = new TableColumn(i);
						if (spreadsheet)
							if (i < 2) {
								newColumn.setPreferredWidth(columnSize[i]);
								newColumn
										.setCellRenderer(new ColorTableCellRenderer());
							} else {
								newColumn.setPreferredWidth(64);
								newColumn
										.setCellRenderer(new NumbersRenderer());
							}
						else {
							newColumn.setPreferredWidth(cellSize.width);
						}
						addColumn(newColumn);
					}
				}

			};
		};
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setRowSelectionAllowed(true);
		table.setColumnSelectionAllowed(false);

		table.setDefaultRenderer(IAtomContainer.class,
				new MoleculeGridCellRenderer(cellSize));
		table.setDefaultRenderer(IMolecule.class, new MoleculeGridCellRenderer(
				cellSize));
		table.setDefaultRenderer(Image.class, new ImageCellRenderer());

		table.setPreferredScrollableViewportSize(new Dimension(
				cellSize.width * 3, (cellSize.height + 30) * 2));
		return table;
	}

	public synchronized JTable getTable() {
		return table;
	}

	public synchronized void setTable(JTable table) {
		this.table = table;
	}

	public void propertyChange(PropertyChangeEvent evt) {

		if (evt instanceof ViewModeChangeEvent) {
			setSpreadsheet(((ViewModeChangeEvent) evt).isSpreadsheet());
		} else if ((evt.getNewValue() != null)
				&& (evt.getNewValue() instanceof SelectedItem)) {
			RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
			try {

				//((SelectedItem) evt.getNewValue()).getDataset();
				
				Subset subset = ((SelectedItem) evt.getNewValue()).getSubset();
                
				model.setSubset(subset);
				label.setText(model.toString());
                label.setToolTipText(label.getText());
				goToRecord(((SelectedItem) evt.getNewValue()).getIndex());

			} catch (SubsetException x) {
				x.printStackTrace();

			}
		} else if (evt.getPropertyName().equals(IChemCastObject.p_properties)) {
			((RandomAccessSubsetTableModel) table.getModel()).propertyChange(evt);
		}
	}
    protected void changeView() {
        //TODO<<<<<<<<<<
    }

	protected void editProperties() {
		try {
			RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
			IAtomContainersList list = model.getSubset().getData();
			int record = table.getSelectedRow();
			list.setSelectedIndex(record);
			MoleculeEditAction ma = new MoleculeEditAction(null,list, null);
			ma.actionPerformed(null);
			model.setTable(model.isTable());
			/*
			if (list instanceof IMoleculesIterator) {
				int record = table.getSelectedRow();
				if (record >=0) {
					((IMoleculesIterator) list).removeAtomContainer(record);
					//update
					model.setTable(model.isTable());
				}
			} else JOptionPane.showMessageDialog(this, "This dataset is read only!");
			*/
				
		} catch (Exception x) {
			JOptionPane.showMessageDialog(this, x);
		}
	}	
	
    protected void info() {
        Subset subset = ((RandomAccessSubsetTableModel) table.getModel()).getSubset();
        try {
            subset.getEditor().view(this, false,"");
        } catch (Exception x) {
            
        }
    }
    protected void save() {
        Subset subset = ((RandomAccessSubsetTableModel) table.getModel()).getSubset();
        if (subset == null) {
            return;
        }
        ArrayList fields = ((RandomAccessSubsetTableModel) table.getModel()).getProperties();
        File file = MyIOUtilities.selectFile(this,null,
                "",
                FileInputState.extensions,FileInputState.extensionDescription,false);
        
        
        if (file == null) return;
        
        firePropertyChange(StatusBar.Message_Status,null,"Saving to file "+file.getAbsolutePath());
        firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
        try {
            
            IChemObjectWriter writer = FileOutputState.getWriter(new FileOutputStream(file), file.getName());
            subset.write(writer, fields);
            writer.close();
        } catch (Exception x) {
            firePropertyChange(StatusBar.Message_Error,null,x.getMessage());           
        }
    }
        
	protected void editAtomContainter() {
		try {
			RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
            if (model.getRowCount() == 0) return;
			IAtomContainersList list = model.getSubset().getData();
			int record = table.getSelectedRow();
            if (record ==-1) {
                JOptionPane.showMessageDialog(this, "No record selected! Click on a record to select it.");
                return;
            }
			list.setSelectedIndex(record);
			MoleculeEditAction ma = new MoleculeEditAction(null,list, null);
			ma.actionPerformed(null);
			model.setTable(model.isTable());
            goToRecord(record);
			/*
			if (list instanceof IMoleculesIterator) {
				int record = table.getSelectedRow();
				if (record >=0) {
					((IMoleculesIterator) list).removeAtomContainer(record);
					//update
					model.setTable(model.isTable());
				}
			} else JOptionPane.showMessageDialog(this, "This dataset is read only!");
			*/
				
		} catch (Exception x) {
			JOptionPane.showMessageDialog(this, x);
		}
	}	
	protected void deleteAtomContainter() {
		try {
			RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
            if (model.getRowCount() == 0) return;
			IAtomContainersList list = model.getSubset().getData();
			if (list instanceof IMoleculesIterator) {
				int record = table.getSelectedRow();
				if (record >=0) {
					((IMoleculesIterator) list).removeAtomContainer(record);
					//update
					model.setTable(model.isTable());
                    goToRecord(record-1);
				}
			} else JOptionPane.showMessageDialog(this, "This dataset is read only!");
				
		} catch (Exception x) {
			JOptionPane.showMessageDialog(this, x);
		}
	}
	protected void addAtomContainter() {
		try {
			RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
            //if (model.getRowCount() == 0) return; //why?????????????
            if (model.getSubset()==null) return;
			IAtomContainersList list = model.getSubset().getData();
			if (list instanceof IMoleculesIterator) {
				
            	int i = JOptionPane.showOptionDialog(this, "Create new structure", "", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null,
        				new String[] {"SMILES","IUPAC name","Structure"}, 
        				"");
            	switch (i) {
            	case 0: {
            		String s = JOptionPane.showInputDialog(this,"Enter SMILES","",JOptionPane.PLAIN_MESSAGE);
            		if (s!=null) try {
	            		SmilesParser p = new SmilesParser();
	            		((IMoleculesIterator) list).addAtomContainer(p.parseSmiles(s));
            		} catch (Exception x) {
            			((IMoleculesIterator) list).addAtomContainer(MoleculeFactory.makeAlkane(1));	
            		}
            		break;
            	} 
            	case 1: {
            		String s = JOptionPane.showInputDialog(this,"Enter IUPAC name","",JOptionPane.PLAIN_MESSAGE);
            		if (s!=null) try {
            			NameToStructure nameToStructure = NameToStructure.getInstance();
            			//Element cmlElement = nameToStructure.parseToCML("acetonitrile");
            			Element cmlElement = nameToStructure.parseToCML(s);
            			IMolecule mol = Compound.readMolecule(cmlElement.toXML());
            			mol.setProperty(CDKConstants.NAMES,s);
            			((IMoleculesIterator) list).addAtomContainer(mol);
            			break;
            		} catch (Exception x) {
                		((IMoleculesIterator) list).addAtomContainer(MoleculeFactory.makeAlkane(1));
                		break;            			
            		}
            	}
            	default: {
            		((IMoleculesIterator) list).addAtomContainer(MoleculeFactory.makeAlkane(0));
            		table.setRowSelectionInterval(list.getAtomContainerCount()-1, list.getAtomContainerCount()-1);
            		editAtomContainter();
            		break;
            	}	
            	}
				
                
				//update
				model.setTable(model.isTable());
                goToRecord(list.getAtomContainerCount()-1);
			} else JOptionPane.showMessageDialog(this, "This dataset is read only!");
				
		} catch (Exception x) {
			JOptionPane.showMessageDialog(this, x);
		}
	}
	protected void first() {
		try {
			goToRecord(0);
		} catch (Exception x) {
		}
	}
    protected void prev() {
        try {
            RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
            if (model.getRowCount() > 0) 
                goToRecord(model.getSubset().getData().getSelectedIndex()-1);

        } catch (Exception x) {
        }
    }

    protected void next() {
        try {
            RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
            if (model.getRowCount() > 0) 
                goToRecord(model.getSubset().getData().getSelectedIndex()+1);            
        } catch (Exception x) {
        }
    }
    
	protected void last() {
		try {
			goToRecord(table.getRowCount() - 1);
		} catch (Exception x) {
		}
	}

	protected void gotoRecord() {
		String s = JOptionPane.showInputDialog(this,"Record number","Go to record",JOptionPane.PLAIN_MESSAGE);
		if ((s != null) && (s.length() > 0)) try {
		    int record = Integer.parseInt(s);
		    goToRecord(record-1);
		} catch (Exception x) {
			
		}
	}
	protected void lookup(String lookupProperty) {
		String s = JOptionPane.showInputDialog(this,"Search by "+lookupProperty,lookupProperty,JOptionPane.PLAIN_MESSAGE);
		if ((s != null) && (s.length() > 0)) try {
			RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
			int record = model.getSubset().getData().indexOf(lookupProperty, s);
		    goToRecord(record);
		} catch (Exception x) {
			
		}			
	}

	protected void goToRecord(int record) throws SubsetException {
        if (record < 0) return;
        RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
		if (model.getRowCount() > 0) {
			model.getSubset().getData().setSelectedIndex(record);
			table.setRowSelectionInterval(record, record);
			table.scrollRectToVisible(table.getCellRect(record, 0, true));
		}

	}

	protected void findSimilar(String lookupProperty) throws SubsetException {
        RandomAccessSubsetTableModel model = (RandomAccessSubsetTableModel) table.getModel();
		if (model.getRowCount() > 0) {
			int row = model.getSubset().getData().getSelectedIndex();
			
			MatrixToolbar toolbar = new MatrixToolbar(null,true,true);
			toolbar.setRowStructure(model.getSubset().getData().getAtomContainer(row));
			toolbar.setColStructure(model.getSubset().getData().getAtomContainer(row));
			JOptionPane.showConfirmDialog(this,toolbar);
		}		
		
	}

	
	public synchronized boolean isSpreadsheet() {
		return spreadsheet;
	}

	public synchronized void setSpreadsheet(boolean spreadsheet) {
		int row = table.getSelectedRow();
		this.spreadsheet = spreadsheet;
		((RandomAccessSubsetTableModel) table.getModel()).setTable(spreadsheet);
		if (spreadsheet) {
			table.setRowHeight(18);
		} else {
			table.setRowHeight(cellSize.height + 24);
			table.setDefaultRenderer(Image.class, new ImageCellRenderer());
		}
		if (row > 0) {
			table.setRowSelectionInterval(row, row);
			table.scrollRectToVisible(table.getCellRect(row, 0, true));
		}
	}
}

class ColorTableCellRenderer extends DefaultTableCellRenderer {
	/**
     * 
     */
    private static final long serialVersionUID = -8469636566374238630L;
    protected Color oddColor = new Color(240, 240, 240);

	public ColorTableCellRenderer() {
		super();
		setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		Component c = super.getTableCellRendererComponent(table, value,
				isSelected, hasFocus, row, column);
		if (isSelected)
			c.setBackground(table.getSelectionBackground());
		else if ((row % 2) == 0) {
			c.setBackground(Color.white);
		} else
			c.setBackground(oddColor);
		if (value != null) {

			setToolTipText(table.getColumnName(column) + "=" + value.toString());
		} else
			setToolTipText("");
		return c;
	}
}

class NumbersRenderer extends ColorTableCellRenderer {
	protected NumberFormat nf = NumberFormat.getInstance();

	public NumbersRenderer() {
		super();
		setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
		nf.setMaximumFractionDigits(4);
	}

	public void setValue(Object value) {
		if ((value != null) && (value instanceof Number)) {
			setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
			Number numberValue = (Number) value;
			value = nf.format(numberValue.doubleValue());
		}
		super.setValue(value);
	}

}

abstract class  MoleculesTableAction extends AbstractAction {
        public MoleculesTableAction(String caption, Icon icon, String hint) {
            super(caption,icon);
            putValue(AbstractAction.SHORT_DESCRIPTION, hint);
        }
}