/* ObjectPropertyFilterEditor.java
 * Author: Nina Jeliazkova
 * Date: Feb 17, 2007 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import chemCast.data.ObjectPropertyFilter;

public class ObjectPropertyFilterEditor extends AbstractFilterEditor {
    /**
     * 
     */
    private static final long serialVersionUID = 4964600959825654563L;
    protected JLabel keyLabel;
    protected JComboBox keyField;
    protected JLabel valueLabel;
    protected JComboBox valueField;
    protected JCheckBox updateValues;
    protected IKeyValuesUpdater keyValuesUpdater;
    public ObjectPropertyFilterEditor(String title,  ObjectPropertyFilter filter) {
        this(new FlowLayout(),title,filter);
    }
    public ObjectPropertyFilterEditor(LayoutManager layout,
                    String title,  
                    ObjectPropertyFilter filter) {
        super(layout,title,filter);
        setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
        Object key = filter.getProperty(ObjectPropertyFilter.p_tag);
        if (key == null) filter.setProperty(ObjectPropertyFilter.p_tag,""); key = "";
        keyField = new JComboBox(new String[] {key.toString()});
        keyField.setSelectedIndex(0);
        keyField.addActionListener(this);
        keyField.addFocusListener(this);
        keyField.setPreferredSize(new Dimension(100,24));
        keyField.setMaximumSize(new Dimension(Integer.MAX_VALUE,24));
        keyLabel = new JLabel("Parameter");
        add(keyLabel);
        add(keyField);
        updateValues = new JCheckBox("Update values");
        updateValues.setSelected(true);
        updateValues.setToolTipText("Retrieves avaiilable values for the selected field. Uncheck if update is slow (there is a limit of 100 values)");
        add(updateValues);
        Object value = filter.getProperty(ObjectPropertyFilter.p_value);
        if (value ==null) value = "";
        valueField = new JComboBox(new String[] {value.toString()});
        valueField.setPreferredSize(new Dimension(100,24));
        valueField.setMaximumSize(new Dimension(Integer.MAX_VALUE,24));        
        valueField.setSelectedIndex(0);
        valueField.addFocusListener(this);
        valueField.addActionListener(this);
        //valueField.setPreferredSize(new Dimension(100,24));
        valueLabel = new JLabel("Value");
        add(valueLabel);
        add(valueField);
 
        setBorder(BorderFactory.createTitledBorder("Define group of compounds"));
    }
    
    @Override
    public void setEditable(boolean editable) {
        super.setEditable(editable);
        keyField.setEditable(editable);
        valueField.setEditable(editable);
    }
    protected void update(Object source) throws Exception {
        if (source == keyField) {
            ((ObjectPropertyFilter) object).setProperty(ObjectPropertyFilter.p_tag, keyField.getSelectedItem().toString());
            if (updateValues.isSelected() && (keyValuesUpdater!=null)) 
                populateValues(keyValuesUpdater.getValuesByField(keyField.getSelectedItem().toString()));
                
        } else if (source == valueField) {
            ((ObjectPropertyFilter) object).setProperty(ObjectPropertyFilter.p_value, valueField.getSelectedItem().toString());
        }
        
    }
    @Override
    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        try {
            update(e.getSource());
        } catch (Exception x) {
            JOptionPane.showMessageDialog(this,x.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
        }
        
    }
    
    @Override
    public void focusLost(FocusEvent e) {
        super.focusLost(e);
        try {
            update(e.getSource());
            } catch (Exception x) {
                JOptionPane.showMessageDialog(this,x.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
            }
    }
    
    public void addTags(Object[] tags) {
        //keyField.removeAllItems();
        keyField.setModel( new DefaultComboBoxModel(tags) );
    }
    
    
    public void populateValues(Collection<Comparable> values) {
        if (values == null) return;
        valueField.setModel( new DefaultComboBoxModel(values.toArray()) ); 
        try {
            valueField.setSelectedIndex(0);
        } catch (Exception x) {
            
        }
    }
    public synchronized IKeyValuesUpdater getKeyValuesUpdater() {
        return keyValuesUpdater;
    }
    public synchronized void setKeyValuesUpdater(IKeyValuesUpdater keyValuesUpdater) {
        this.keyValuesUpdater = keyValuesUpdater;
    }
    

}

