/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.text.NumberFormat;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import org.openscience.cdk.interfaces.IAtomContainer;

import ambit.io.MyIOUtilities;
import ambit.processors.results.AbstractSimilarityMatrix;
import ambit.processors.results.LightSimilarityMatrix;
import ambit.processors.results.SimilarityMatrix;
import ambit.similarity.ISimilarityProcessor;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.data.PairwiseSimilarityFilter;
import chemCast.data.SelectedItem;
import chemCast.events.SelectItemEvent;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;
import chemCast.weka.ISubsetsModel;

public class SimilarityMatrixPanel extends JPanel implements PropertyChangeListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 141147817785849252L;
	protected JTable table;
	//protected EditorPanel border;
	protected JLabel processorTitle;
	protected JComboBox rowsBox;
	protected JComboBox columnBox;
	protected int cellSize = 12;
	protected Subsets subsetsRow = null;
	protected Subset subsetRow = null;
	protected Subsets subsetsCol = null;
	protected Subset subsetCol = null;
    private int currentRow = 0;
    private int currentColumn = 0;
	protected LightSimilarityMatrix matrix;
	protected PairwiseSimilarityFilter filter = null;
    protected Subset selectedSubsetRows = null;
    protected Subset selectedSubsetColumns = null;
    protected MatrixToolbar matrixToolbar;
    
	public SimilarityMatrixPanel() {
		super(new BorderLayout());
		matrix = new LightSimilarityMatrix(null) {
		    @Override
		    public float calcValueAt(int row, int column) throws Exception {
		        if (processor instanceof ISubsetsModel) 
                    return ((ISubsetsModel) processor).predict(rows,row,columns,column);
                else return super.calcValueAt(row, column);
		    }
        };
		createWidgets();
		setModel(matrix);
	}
	public void setModel(AbstractTableModel model) {
		table.setModel(model);
	}
	protected void createWidgets() {
		
		processorTitle = new JLabel();
        table = new JTable() {
        	public void createDefaultColumnsFromModel() {
        		
        		TableModel model = getModel();
                // Remove any current columns
                TableColumnModel cm = getColumnModel();
                while (cm.getColumnCount() > 0) {
                          cm.removeColumn(cm.getColumn(0));
                }
        		
        		TableCellRenderer r = new MatrixCellRenderer();
                for (int i=0; i < model.getColumnCount();i++) {
                    TableColumn c = new TableColumn(i);
                    c.setMinWidth(cellSize);
                    c.setMaxWidth(cellSize);
                    c.setHeaderValue(new Integer(i));
                    c.setPreferredWidth(cellSize);
                    c.setCellRenderer(r);
                    addColumn(c);
                }
        	}
        };
        JScrollPane p = new JScrollPane(table);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF); 
        table.setRowHeight(cellSize);
        table.setShowGrid(true);
        table.setGridColor(Color.white);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.getTableHeader().setReorderingAllowed(false);
        table.setSurrendersFocusOnKeystroke(true);
        table.setRowSelectionAllowed(false);
        table.setColumnSelectionAllowed(false);
        table.setCellSelectionEnabled(true);
        table.addMouseListener(new MouseAdapter() {
        	@Override
        	public void mouseClicked(MouseEvent e) {
    	    	Point p = e.getPoint();
    	    	int r = table.rowAtPoint(p);
    	    	int c = table.columnAtPoint(p);
       	    	if (!e.isMetaDown()) {
        	    	if ((r >= 0) && (subsetRow!=null)) 
        	    		subsetsRow.firePropertyChange(subsetsRow.getType(),
            					null,new SelectedItem(subsetsRow,subsetRow,r));
        	    		
	   	    	} else {
	        	    	if ((c >=0) && (subsetsCol!=null)) 
	    					subsetsCol.firePropertyChange(subsetsCol.getType(),
	        					null,new SelectedItem(subsetsCol,subsetCol,c));
	   	    	}
       	    	setRow(r);
       	    	setColumn(c);
       	    	/*
    	    	matrixToolbar.setRowNumber(getRow()+1);
    	    	matrixToolbar.setColNumber(getColumn()+1);
    	    	*/    
    	    	matrixToolbar.setSimilarity(table.getValueAt(getRow(), getColumn()));
    	    	
    	    	if ((getRow() >= 0) && (subsetRow!=null)) {
    	    		try {
    	    			matrixToolbar.setRowStructure(subsetRow.getData().getAtomContainer(getRow()));
    	    		} catch (Exception x) {
    	    			matrixToolbar.setRowStructure(null);
    	    		}
    	    	} else matrixToolbar.setRowStructure(null);
    	    	
    	    	if ((getColumn() >=0) && (subsetsCol!=null)) {
					try {
						matrixToolbar.setColStructure(subsetCol.getData().getAtomContainer(getColumn()));
					} catch (Exception x) {
						matrixToolbar.setColStructure(null);
					}
    	    	} else 	matrixToolbar.setColStructure(null);
    	    	


        	}
        });

        add(p,BorderLayout.CENTER);
        
        JToolBar topPanel = new JToolBar();
        topPanel.setFloatable(false);

        
		rowsBox = new JComboBox(ChemCASTModule.datasetName);
       	rowsBox.setSelectedIndex(0);
       	rowsBox.setToolTipText("Click on the matrix to see the compound at particular row");
       	rowsBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
        		String c = (String)cb.getSelectedItem();
        		setRowDataset(null,null);
			}
		});
		columnBox = new JComboBox(ChemCASTModule.datasetName);
		columnBox.setSelectedIndex(0);
		columnBox.setToolTipText("Right click on the matrix to see the compound at particular column");
		columnBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setColDataset(null,null);

			}
		});
		JButton saveMatrix = new JButton(new AbstractAction("Save matrix") {
			public void actionPerformed(ActionEvent arg0) {
				saveMatrix();
			}
			
		});
	
		JLabel rowsLabel = new JLabel("Rows ");
		JLabel columnsLabel = new JLabel(" Columns ");
		topPanel.add(rowsLabel);
		topPanel.add(rowsBox);
		topPanel.add(columnsLabel);
		topPanel.add(columnBox);
		topPanel.add(saveMatrix);
		
		topPanel.add(new JButton(new AbstractAction("Zoom in") {
			public void actionPerformed(ActionEvent arg0) {
				cellSize *= 1.5;
				table.setRowHeight(cellSize);
				matrix.fireTableStructureChanged();
			}
		}));	
		topPanel.add(new JButton(new AbstractAction("Zoom out") {
			public void actionPerformed(ActionEvent arg0) {
				cellSize = cellSize *2 /3;
				if (cellSize<2) cellSize = 12;
				table.setRowHeight(cellSize);
				matrix.fireTableStructureChanged();
			}
		}));			
		add(topPanel,BorderLayout.NORTH);
		
		add(processorTitle,BorderLayout.SOUTH);
		Action[] actions = new Action[] {
			new AbstractAction("Find") {
			    public void actionPerformed(ActionEvent e) {
	                setEnabled(false);
	                try {
	                	showMostSimilar(subsetsRow, subsetRow, getRow(),subsetsCol,subsetCol, getColumn());
	                } catch (Exception x) {
	                	JOptionPane.showMessageDialog((Component)e.getSource(),x.getMessage());
	                }	
	                setEnabled(true);
			        
			    }
	        }
			/*
			,
			new AbstractAction("Filter") {
			    public void actionPerformed(ActionEvent e) {
	                setEnabled(false);
	                try {
	                	filterMostSimilar(subsetsRow, subsetRow, getRow(),subsetsCol,subsetCol, getColumn());
	                } catch (Exception x) {
	                	JOptionPane.showMessageDialog((Component)e.getSource(),x.getMessage());
	                }	
	                setEnabled(true);
			        
			    }
	        }		
	        */	
		};
        matrixToolbar = new MatrixToolbar(actions);
		add(matrixToolbar, BorderLayout.EAST);
	}
    public void addListSelectionListener(ListSelectionListener listener) {
        table.getSelectionModel().addListSelectionListener(listener);
    }
    public void removeListSelectionListener(ListSelectionListener listener) {
        table.getSelectionModel().removeListSelectionListener(listener);
    }
	public void propertyChange(PropertyChangeEvent e) {
		Object result = e.getNewValue();
		if (result == null) return;
		else
		if (StatusBar.Message_Processing.equals(e.getPropertyName())) {
			if ((Boolean)result) {
				matrix.setEnabled((Boolean)result);
                matrixToolbar.setEnabled((Boolean)result);
			}
		} else
    	if (result instanceof AbstractSimilarityMatrix) {
    		setModel((AbstractSimilarityMatrix) result);
    	} else
        if (result instanceof ISimilarityProcessor) {
        	matrix.setProcessor((ISimilarityProcessor) result);
            
            processorTitle.setText("<html><b>" + result.toString() + "</b></html>");

        } else    		
    	if (result instanceof SelectedItem) {	
    		setDataset(((SelectedItem)result).getDataset(),((SelectedItem)result).getSubset());

    	}
	}
	public void setRowDataset(Subsets datasets, Subset subset) {
		matrixToolbar.setRowDatasetName("");
    	LightSimilarityMatrix matrix = null;
		if (table.getModel() instanceof LightSimilarityMatrix) 
			 matrix = ((LightSimilarityMatrix) table.getModel()); 
		else return;	
		try {
			subsetsRow = datasets;
			subsetRow = subset;
			if (subset != null) {
				matrix.setRows(subset.getData());
				matrixToolbar.setRowDatasetName(datasets.getName() + " [" +subset.getName() + "]");
			}
		}  catch (Exception x) {
			subsetsRow = null;
			subsetRow = null;	
			matrix.setRows(null);
			matrixToolbar.setRowDatasetName("");
		}
		setRow(-1);
	}
	public void setColDataset(Subsets datasets, Subset subset) {
		System.out.println("set col dataset");
		matrixToolbar.setColumnDatasetName("");
    	LightSimilarityMatrix matrix = null;
		if (table.getModel() instanceof LightSimilarityMatrix) 
			 matrix = ((LightSimilarityMatrix) table.getModel()); 
		else return;	
		try {
			subsetsCol = datasets;
			subsetCol = subset;
			if (subset != null) {
				matrix.setColumns(subset.getData());
				matrixToolbar.setColumnDatasetName(datasets.getName() + " [" +subset.getName() + "]");
			}
		} catch (Exception x) {
			subsetsCol = null;
			subsetCol = null;	
			matrix.setColumns(null);
			matrixToolbar.setColumnDatasetName("");
		}
		setColumn(-1);
		
	}
    public void setDataset(Subsets datasets, Subset subset) {
		if (datasets != null) {
			if (datasets.getType().equals(rowsBox.getSelectedItem())) {
				matrixToolbar.setRowStructure(null);
				matrixToolbar.setRowNumber(-1);
				setRowDataset(datasets,subset);
			}	
			if (datasets.getType().equals(columnBox.getSelectedItem())) {
				matrixToolbar.setColStructure(null);
				matrixToolbar.setColNumber(-1);
				setColDataset(datasets,subset);
			}	
		} else {
			setRow(-1);
			setColumn(-1);
			matrixToolbar.setRowStructure(null);
			matrixToolbar.setColStructure(null);
			setRowDataset(null,null);
			setColDataset(null,null);
		}
    }
	public JTable getTable() {
		return table;
	}
    
	public void saveMatrix() {
		File file = MyIOUtilities.selectFile(this, "Save ", System.getProperty("user.dir"),
				new String[] {".png",".csv"},
				new String[] {"Image (*.png)","Comma delimited file (*.csv)"},
				false);
		if (file ==null) return;
	    try {
	    	String fn = file.getName().toLowerCase(); 
	    	if (fn.endsWith(".png")) {
	    		ImageIO.write(matrix.toImage(), "png", file);
	    	} else if (fn.endsWith(".csv")) {
	    		FileOutputStream out = new FileOutputStream(file);
	    		matrix.write(new OutputStreamWriter(out));
	    		out.close();
	    	} else 
                JOptionPane.showMessageDialog(this,"Unsupported format");
/*
                if (fn.endsWith(".xls")) {
                
	    		HSSFWorkbook wb = new HSSFWorkbook();
	    	    FileOutputStream fileOut = new FileOutputStream(file);

	    	    HSSFSheet sheet = wb.createSheet("Similarity matrix");

//	    		 Create the drawing patriarch.  This is the top level container for
	    	    // all shapes. This will clear out any existing shapes for that sheet.
	    	    HSSFPatriarch patriarch = sheet.createDrawingPatriarch();

	    	    HSSFClientAnchor anchor;
	    	    anchor = new HSSFClientAnchor(0,0,0,255,(short)2,2,(short)4,7);
	    	    anchor.setAnchorType( 2 );
	    	    patriarch.createPicture(arg0, arg1)createPicture(anchor, matrix.toImage());
	    	    
	    	    wb.write(fileOut);
	    	    fileOut.close();
            
    	    	}
            */
	    } catch (Exception x) {
	    	x.printStackTrace();
	    }
	}    
    public void showMostSimilar(Subsets subsetsRow, Subset subsetRow, int row,
    			Subsets subsetsCol,Subset subsetCol, int col) throws SubsetException {

	    IAtomContainer query = null;
	    try {
		      if (matrixToolbar.queryFromRow())
		        	query = subsetRow.getData().getAtomContainer(row);
		      else
		        	query = subsetCol.getData().getAtomContainer(col);
	    } catch (Exception x) {
	        	throw new SubsetException("Structure not defined");
	    }
    	  
	    if (query == null) throw new SubsetException("Structure not defined");
	    try {	      
            matrix.setEnabled(false);
            matrixToolbar.setEnabled(false);
            setEnabled(false);
            
	        if (filter == null) filter = new PairwiseSimilarityFilter();
	        filter.setProcessor(matrix.getProcessor());	        
	        filter.setQuery(query);        	
	        filter.setThreshold(matrixToolbar.getThreshold());
	        filter.setCondition(matrixToolbar.getCondition());
            
	        Subset subset = new Subset("Most similar "+ filter.toString(),null,Color.green);
	        
	        subset.setFilterClass("chemCast.data.ProcessorFilter");
      
            if (matrixToolbar.getNNFromRows()) {
                subsetsRow.firePropertyChange(new SelectItemEvent(
                        this,
                        "",
                        null,
                        new SelectedItem(subsetsRow,null,0)
                        ));
                if (selectedSubsetRows != null) {
                    selectedSubsetRows.clear();
                    selectedSubsetRows = null;
                }                
                subsetsRow.createNewSubset(subset,filter);
                selectedSubsetRows = subset;
            } else {
                subsetsCol.firePropertyChange(new SelectItemEvent(
                        this,
                        "",
                        null,
                        new SelectedItem(subsetsCol,null,0)
                        ));                
                
                
                if (selectedSubsetColumns != null) {
                    selectedSubsetColumns.clear();
                    selectedSubsetColumns = null;
                }
                subsetsCol.createNewSubset(subset,filter);
                selectedSubsetColumns = subset;
                
            }
            matrix.setEnabled(true);
            matrixToolbar.setEnabled(true);
            setEnabled(true);
    	} catch (Exception x) {
            matrix.setEnabled(true);
            matrixToolbar.setEnabled(true);
            setEnabled(true);
    		throw new SubsetException(x.getMessage());
    	}
    }
    
    public void filterMostSimilar(Subsets subsetsRow, Subset subsetRow, int row,
			Subsets subsetsCol,Subset subsetCol, int col) throws SubsetException {

	    IAtomContainer query = null;
	    try {
		      if (matrixToolbar.queryFromRow())
		        	query = subsetRow.getData().getAtomContainer(row);
		      else
		        	query = subsetCol.getData().getAtomContainer(col);
	    } catch (Exception x) {
	        	throw new SubsetException("Structure not defined");
	    }
		  
	    if (query == null) throw new SubsetException("Structure not defined");
	    try {	      
	        matrix.setEnabled(false);
	        matrixToolbar.setEnabled(false);
	        setEnabled(false);
	        
	        if (filter == null) filter = new PairwiseSimilarityFilter();
	        filter.setProcessor(matrix.getProcessor());	        
	        filter.setQuery(query);        	
	        filter.setThreshold(matrixToolbar.getThreshold());
	        filter.setCondition(matrixToolbar.getCondition());
	        
	        Subset subset = new Subset("Most similar "+ filter.toString(),null,Color.green);
	        
	        subset.setFilterClass("chemCast.data.ProcessorFilter");
	  
	        if (matrixToolbar.getNNFromRows()) {
	            subsetsRow.firePropertyChange(new SelectItemEvent(
	                    this,
	                    "",
	                    null,
	                    new SelectedItem(subsetsRow,null,0)
	                    ));
	            if (selectedSubsetRows != null) {
	                selectedSubsetRows.clear();
	                selectedSubsetRows = null;
	            }                
	            subsetsRow.createNewSubset(subset,filter);
	            selectedSubsetRows = subset;
	        } else {
	            subsetsCol.firePropertyChange(new SelectItemEvent(
	                    this,
	                    "",
	                    null,
	                    new SelectedItem(subsetsCol,null,0)
	                    ));                
	            
	            
	            if (selectedSubsetColumns != null) {
	                selectedSubsetColumns.clear();
	                selectedSubsetColumns = null;
	            }
	            subsetsCol.createNewSubset(subset,filter);
	            selectedSubsetColumns = subset;
	            
	        }
	        matrix.setEnabled(true);
	        matrixToolbar.setEnabled(true);
	        setEnabled(true);
		} catch (Exception x) {
	        matrix.setEnabled(true);
	        matrixToolbar.setEnabled(true);
	        setEnabled(true);
			throw new SubsetException(x.getMessage());
		}
    }    
	public int getColumn() {
		return currentColumn;
	}
	public void setColumn(int column) {
		this.currentColumn = column;
    	matrixToolbar.setColNumber(column+1);

	}
	public int getRow() {
		return currentRow;
	}
	public void setRow(int row) {
		this.currentRow = row;
    	matrixToolbar.setRowNumber(row+1);

	}
}


class MatrixCellRenderer extends DefaultTableCellRenderer {
	static Border b = BorderFactory.createMatteBorder(1,1,0,0,Color.white);
    static Border sb = BorderFactory.createMatteBorder(2,2,2,2,Color.blue);
    protected static NumberFormat nf = NumberFormat.getNumberInstance();
	/**
	 * 
	 */
	private static final long serialVersionUID = -76750530300824199L;
	public MatrixCellRenderer() {
		super();
		nf.setMaximumFractionDigits(3);
	}
    /* (non-Javadoc)
     * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
     */
    public Component getTableCellRendererComponent(JTable table,
            Object value, boolean isSelected, boolean hasFocus,
            int row, int column) {
        // TODO Auto-generated method stub
        Component c = super.getTableCellRendererComponent(table, value,
                isSelected, hasFocus, row, column);
        if (value instanceof Number) {
	        Color clr = SimilarityMatrix.valueToColor((Number)value);
	        setBackground(clr);
	        setForeground(clr);
	        setToolTipText("("+(row+1)+ "," + (column+1) + ")=" + nf.format(((Number)value).doubleValue()));
        }
        /*
        if (isSelected)
            setBorder(sb);
        else
            setBorder(b);
            */
        return c;
    }
    
}



