/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.SubCategoryAxis;
import org.jfree.chart.entity.CategoryItemEntity;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.GroupedStackedBarRenderer;
import org.jfree.data.KeyToGroupMap;
import org.jfree.data.category.DefaultCategoryDataset;

import ambit.similarity.IModelHistogram;
import ambit.similarity.SimilarityHistogram;
import ambit.stats.datastructures.Bin;
import ambit.stats.datastructures.Histogram;
import ambit.stats.datastructures.SelfBin;
import ambit.stats.datastructures.SubcategoryBin;
import ambit.stats.datastructures.ValuePair;
import ambit.ui.UITools;
import ambit.ui.actions.AbstractActionWithTooltip;
import chemCast.similarity.data.SimilaritySubsets;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

public class SimilarityResultViewer extends JPanel 
					 implements PropertyChangeListener, ChartMouseListener  
															{
	/**
	 * 
	 */
	private static final long serialVersionUID = 8438818783583582287L;
	//protected SimilarityMatrixPanel matrix;
	protected ChartPanel histogram;
	protected JFreeChart histogramChart;
	protected String title;
    protected SimilaritySubsets subsets;
    protected CategoryAxis domainAxis;
    protected SubCategoryAxis stackedAxis;
    protected GroupedStackedBarRenderer stackedRenderer;
    protected BarRenderer barRenderer;
    protected JToolBar histogramModeToolbar;
    protected int histogramMode = 0;
    protected IModelHistogram result;
	
	public IModelHistogram getResult() {
		return result;
	}

	public void setResult(IModelHistogram result) {
		this.result = result;
	}

	public SimilarityResultViewer(String title, SimilaritySubsets subsets) {
		super();
		
		histogramModeToolbar = new JToolBar();
		ButtonGroup bg = new ButtonGroup();
		
		JToggleButton button_mode0 = new JToggleButton(new AbstractActionWithTooltip("",UITools.createImageIcon("chemCast/ui/images/histogram_distances.png"),"Distribution of similarity values") {
			public void actionPerformed(ActionEvent arg0) {
				if (((JToggleButton)arg0.getSource()).isSelected())
					setHistogramMode(0);
				else 
					setHistogramMode(1);
				
			}
		});
		bg.add(button_mode0);
		button_mode0.setSelected(true);
		histogramModeToolbar.add(button_mode0);
		JToggleButton button_mode1 = new JToggleButton(new AbstractActionWithTooltip("",UITools.createImageIcon("chemCast/ui/images/histogram_stacked.png"),"Predicted groups histogram") {
			public void actionPerformed(ActionEvent arg0) {
				if (((JToggleButton)arg0.getSource()).isSelected())
					setHistogramMode(1);
				else 
					setHistogramMode(0);
			}
		});
		histogramModeToolbar.add(button_mode1);
		bg.add(button_mode1);
		/*
		histogramModeToolbar = new JComboBox(new String[] {"Similarity histogram","Predicted groups histogram"});
		histogramMode.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (getResult() != null) {
	                CategoryPlot plot = ((CategoryPlot) histogramChart.getPlot());
	                plot.getDomainAxis().setLabel(getResult().toString());
	                DefaultCategoryDataset dataset = (DefaultCategoryDataset) plot.getDataset();
	                dataset.clear();
	                updateCategoryDataset(dataset,getResult(),plot);
				}
			}
		});
		*/
		setLayout(new BorderLayout());
		this.title = "Similarity to " + title;
		histogram = new ChartPanel(null);
		add(histogram,BorderLayout.CENTER);
		add(histogramModeToolbar,BorderLayout.NORTH);
		histogram.setPreferredSize(new Dimension(300,200));
		setPreferredSize(new Dimension(300,300));
		histogramChart = null;
        this.subsets = subsets;
	}

	public void propertyChange(PropertyChangeEvent e) {
		Object result = e.getNewValue();

		if (result == null) {
			 if (histogramChart != null) {
	                CategoryPlot plot = ((CategoryPlot) histogramChart.getPlot());
	                DefaultCategoryDataset dataset = (DefaultCategoryDataset) plot.getDataset();
	                dataset.clear();
	                dataset.addValue(0,"","");
	                plot.getDomainAxis().setLabel("");
			 }
		}
		else if (result instanceof SimilarityHistogram) {
    		
    		SimilarityHistogram fp = (SimilarityHistogram) result;
            double[] bins = fp.getBins();
            String caption = fp.toString();
			if (histogramChart == null) {
                DefaultCategoryDataset dataset = new DefaultCategoryDataset();
                for (int i=0; i < bins.length;i++) {
                    dataset.addValue(fp.getCounter()[i],caption, new Double(bins[i]));
                }                
				histogramChart = ChartFactory.createBarChart3D(
						title,
						fp.getTitle(), 
						"Number of compounds", 
						dataset, 
						PlotOrientation.VERTICAL,
						true, false, false);
				histogramChart.setBackgroundPaint(Color.white);
				histogramChart.setAntiAlias(true);
				histogram.setChart(histogramChart);
				domainAxis = ((CategoryPlot) histogramChart.getPlot()).getDomainAxis();
				
				stackedAxis = null;
			} else {
				CategoryPlot plot = ((CategoryPlot) histogramChart.getPlot());
                DefaultCategoryDataset dataset = (DefaultCategoryDataset) plot.getDataset();
                for (int i=0; i < bins.length;i++) {
                    dataset.addValue(fp.getCounter()[i],caption, new Double(bins[i]));
                }  
			}
    	} else if (result instanceof IModelHistogram) {
            IModelHistogram fp = (IModelHistogram) result;
            setResult(fp);
            if (histogramChart == null) {
                DefaultCategoryDataset dataset = new DefaultCategoryDataset();
                
                /*
                for (int i=0; i < bins.length;i++) {
                    dataset.addValue(fp.getCounter()[i],caption, new Double(bins[i]));
                } 
                */               
                histogramChart = ChartFactory.createStackedBarChart(
                        title,
                        fp.toString(), 
                        "Number of compounds", 
                        dataset, 
                        PlotOrientation.VERTICAL,
                        true, false, false);
                
				domainAxis = ((CategoryPlot) histogramChart.getPlot()).getDomainAxis();
				stackedAxis = null;                
                stackedRenderer = new GroupedStackedBarRenderer() {
                 	@Override
                	public Paint getSeriesPaint(int arg0) {
                		Comparable c = getPlot().getDataset().getRowKey(arg0);
                		if (c instanceof Subsets) {
                			return ((Subsets) c).getParentDataset().getColor();
                		} else 
                			return super.getSeriesPaint(arg0);
                	}
                };
                barRenderer = new BarRenderer() {
                	@Override
                	public Paint getSeriesPaint(int arg0) {
                		Comparable c = getPlot().getDataset().getRowKey(arg0);
                		if (c instanceof Subsets) {
                			return ((Subsets) c).getParentDataset().getColor();
                		} else 
                			return super.getSeriesPaint(arg0);
                	}
                	@Override
                	public Paint getSeriesFillPaint(int arg0) {
                		return getSeriesPaint(arg0);
                	}
                };
                ((CategoryPlot) histogramChart.getPlot()).setRenderer(barRenderer);
                updateCategoryDataset(dataset,fp,((CategoryPlot) histogramChart.getPlot()));
                histogramChart.setBackgroundPaint(Color.white);
                histogramChart.setAntiAlias(true);
                histogram.setChart(histogramChart);
                histogram.addChartMouseListener(this);
            } else {
                CategoryPlot plot = ((CategoryPlot) histogramChart.getPlot());
                plot.getDomainAxis().setLabel(fp.toString());
                DefaultCategoryDataset dataset = (DefaultCategoryDataset) plot.getDataset();
                dataset.clear();
                updateCategoryDataset(dataset,fp,plot);        
                
            }
        }
    	//else  		matrix.propertyChange(e);

	}
	protected void updateCategoryDataset(DefaultCategoryDataset dataset, IModelHistogram fp,CategoryPlot plot) {
        Enumeration<Comparable> datasets = fp.getDatasets();
        KeyToGroupMap map = new KeyToGroupMap("");
        
        boolean stacked = false;
        ArrayList<String> categories = new ArrayList<String>();
        
        
        domainAxis.setVisible(false);
        if (stackedAxis != null) stackedAxis.setVisible(false);

        while (datasets.hasMoreElements()) {
        	Comparable datasetTitle = datasets.nextElement();
        	Histogram<Bin> histogram = fp.getHistogram(datasetTitle,histogramMode);
        	//System.out.println(histogram);
        	if (histogram == null) continue;
        	Iterator <Bin> bins = histogram.keySet().iterator();
        	
        	while (bins.hasNext()) {
        		Bin bin = bins.next();
        		
        		Object t = bin.getLow();
        		if (t instanceof ValuePair) {
        			ValuePair p = (ValuePair)t;
        			String s = datasetTitle + "[ " + p.getObserved().toString() + " ]";
        			//System.out.println(p.getPredicted());
        			dataset.addValue(new Integer(histogram.getFrequency(bin)),s,
        					new SubcategoryBin<Comparable>(p.getPredicted(),fp.toString()));
        			map.mapKeyToGroup(s, datasetTitle);
                    stacked = true;
        		} else
        			dataset.addValue(new Integer(histogram.getFrequency(bin)),datasetTitle,bin);
        	}
        	if (stacked)
        		if (categories.indexOf(datasetTitle.toString())==-1)
        			categories.add(datasetTitle.toString());
        }
        
        if (stacked && (plot != null)) {
        	
       		stackedAxis = new SubCategoryAxis("Predicted category");
        	stackedAxis.setCategoryMargin(0.05);

        	Iterator<String> s = categories.iterator();
        	while (s.hasNext())
        		stackedAxis.addSubCategory(s.next());
        	
        	plot.setDomainAxis(stackedAxis);
        	((CategoryPlot) histogramChart.getPlot()).setRenderer(stackedRenderer);
        	stackedRenderer.setSeriesToGroupMap(map);
        	stackedAxis.setVisible(true);
        	
        	plot.setDomainAxis(stackedAxis);
        } else {	
        	plot.setDomainAxis(domainAxis);
            ((CategoryPlot) histogramChart.getPlot()).setRenderer(barRenderer);
            domainAxis.setVisible(true);
        }
       	plot.getDomainAxis().setLabel(fp.toString());
        
        
	}

    public void chartMouseClicked(ChartMouseEvent event) {
        
        
        if (event.getEntity() instanceof CategoryItemEntity) {
            CategoryItemEntity ce = (CategoryItemEntity) event.getEntity();
            if (ce.getCategory() instanceof Bin) {
                try {
                    //setCursor(cursor)
                    CategoryPlot plot = ((CategoryPlot) histogramChart.getPlot());
                    String distance = plot.getDomainAxis().getLabel();
                    Object series = ce.getDataset().getRowKey(ce.getSeries());
                    
                    Bin bin = (Bin) ce.getCategory();
                    //<group name="LogP greater than 6" value="Hydrophobic" tag="LogP" MinValue="6" MaxValue="100000" color="000088"
                    Hashtable t = new Hashtable();
                    t.put("MinValue",bin.getLow().toString());
                    t.put("MaxValue",bin.getHigh().toString());
                    t.put("name", bin.toString());
                    if (distance.startsWith("Verhaar scheme")) 
                        t.put("tag",distance);
                    else if (distance.startsWith("BfRRules"))
                        t.put("tag",distance);
                    else if (ce.getCategory() instanceof SubcategoryBin) {
                    	//t.put("tag",((SubcategoryBin)ce.getCategory()).getTag());
                    	t.put("tag",subsets + "." + ((SubcategoryBin)ce.getCategory()).getTag());
                    } else
                    	t.put("tag",  series + "." + distance);
                    Subset subset = new Subset(ce.getCategory().toString(),t,Color.green);
                    if (ce.getCategory() instanceof SelfBin) {
                        subset.setProperty(Subset.p_value,ce.getCategory().toString());
                        subset.setFilterClass("chemCast.data.PropertyFilter");
                        
                    } else {
                        subset.setFilterClass("chemCast.data.RangePropertyFilter");
                    }
                    subsets.createNewSubset(subset,null);
                } catch (Exception x) {
                    x.printStackTrace();
                }
            } 
            

               /*
            SelectedItem newSelected = new SelectedItem(
                    datasets,
                    datasets.get(xy.getSeriesIndex()),
                    xy.getItem());
            datasets.firePropertyChange(datasets.getName(), 
                    selectedItem, 
                    newSelected
                    );
                    */            
        }
        /*
        if (event.getEntity() instanceof XYItemEntity) {
            XYItemEntity xy = (XYItemEntity) event.getEntity();
            try {
                Subsets datasets = ((MoleculesXYZDataset)xy.getDataset()).getReaders();
                SelectedItem newSelected = new SelectedItem(
                            datasets,
                            datasets.get(xy.getSeriesIndex()),
                            xy.getItem());
                datasets.firePropertyChange(datasets.getName(), 
                        selectedItem, 
                        newSelected
                        );

                setSelected(newSelected);
             } catch (Exception x) {
                
            }
        }    
        */
    }

    public void chartMouseMoved(ChartMouseEvent arg0) {
        // TODO Auto-generated method stub
        
    }

	public int getHistogramMode() {
		return histogramMode;
	}

	public void setHistogramMode(int histogramMode) {
		this.histogramMode = histogramMode;
		if (histogramChart ==null) return;
        CategoryPlot plot = ((CategoryPlot) histogramChart.getPlot());
        if (plot == null) return;
        plot.getDomainAxis().setLabel(getResult().toString());
        DefaultCategoryDataset dataset = (DefaultCategoryDataset) plot.getDataset();
        dataset.clear();
        updateCategoryDataset(dataset,getResult(),plot);		
	}
}


