/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.openscience.cdk.qsar.model.QSARModelException;

import weka.core.Instances;
import weka.gui.GenericObjectEditor;
import weka.gui.PropertyDialog;
import ambit.similarity.DefaultSimilarityProcessor;
import ambit.similarity.DoubleValuesHistogram;
import ambit.similarity.IClassificationModel;
import ambit.similarity.IModelStatistics;
import ambit.similarity.IPredictionModel;
import ambit.similarity.SimilarityProcessorEditor;
import ambit.similarity.StringValuesHistogram;
import ambit.stats.datastructures.Bin;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Subsets;

/**
 * 
 * TODO add description.
 * The logic is as follows:
 * <pre>
 if (useclassproperty) {
    if (classproperty==null) does classification by subsets
    else predicts classproperty by nearest neighbors
} else does clustering by subsets
</pre>
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Feb 4, 2007
 */
public abstract class AbstractWekaModel extends DefaultSimilarityProcessor implements
		InstanceProcessor, ISubsetsModel, IClassificationModel, IPredictionModel {
	protected Subsets datasets = null;
	protected Subsets2Instances convertor = null;
	protected boolean useAtomEnvironments = false;
	protected String predictedProperty = null;
	protected String classificationProperty = "Group";
	protected boolean inProgress = false;

	
	public AbstractWekaModel() {
		super();
		convertor = new Subsets2Instances();
	}

	public Subsets getDatasets() {
		return datasets;
	}

	public void setDatasets(Subsets datasets) {
		this.datasets = datasets;

	}

	public void build() throws QSARModelException {
        setPredicting(false);
		if (datasets == null) throw new QSARModelException("Dataset not assigned!");
		try {
			setInProgress(true);
            int enabledcount = 0; 
            for (int i=0; i < datasets.size();i++)
                if (datasets.get(i).isEnabled()) {
                    enabledcount++;
                }
            if (convertor.isUseClassProperty())
	            if (enabledcount==1) {
	                DatasetDef d = datasets.getParentDataset();
                    if ("".equals(d.getDatasetResultField())) d.setDatasetResultField("Result");
	                convertor.setClassProperty(d.getDatasetResultField());
                    
	                if (predictedProperty == null)
	                	setPredictedProperty(d.getDatasetResultField());
	            } else convertor.setClassProperty(null);
            else convertor.setClassProperty(null);
            
            if (enabledcount ==0) throw new Exception("No dataset selected!");
            
			Instances instances = convertor.properties2Instances(datasets);
			setTrainingPrefix(datasets.toString()); // + getSimilarityProperty();
			
			if (instances.numInstances() ==0) throw new Exception("No data available!");
			build(instances);

            //for (int i=0; i < att)
            setPredicting(true);
		} catch (Exception x) {
            setPredicting(false);
			x.printStackTrace();
			setInProgress(false);
			throw new QSARModelException(x.getMessage());
		}
		setDatasets(null);
		setInProgress(false);
        
	}
	protected abstract void build(Instances instances) throws Exception; 

	public synchronized void predict() throws QSARModelException {
		if (datasets == null) throw new QSARModelException("Dataset not assigned!");
		try {
			setInProgress(true);
			convertor.setInstanceProcessor(this);
			convertor.properties2Instances(datasets);
			setDatasets(null);
			setInProgress(false);
		} catch (Exception x) {
			setDatasets(null);
			setInProgress(false);
			throw new QSARModelException(x.getMessage());
		}
	}
	protected IModelStatistics createModelStatistics(boolean classification,String datasetTitle) {
		IModelStatistics ms = modelStatistics;
		if (classification) {
			StringBuffer property = new StringBuffer();
			property.append(getSimilarityProperty());
			property.append(".");
			property.append(getClassificationProperty());
            if (ms ==null)  ms = new StringValuesHistogram(property.toString());
            else if (ms instanceof StringValuesHistogram) ms.clear(datasetTitle);
            else ms = new StringValuesHistogram(property.toString());
        } else {
            if (ms ==null)  ms = new DoubleValuesHistogram<Bin>(getSimilarityProperty());
            else if (ms instanceof DoubleValuesHistogram) ms.clear(datasetTitle);
            else ms = new DoubleValuesHistogram<Bin>(getSimilarityProperty());            
        }
		return ms;
	}
	public void setParameter(Object parameter, Object value) {
		if (parameter.equals(p_classification_property)) classificationProperty = value.toString();
		else if (parameter.equals(p_predicted_property)) predictedProperty = value.toString();  
		else 
        if (parameter.equals(ChemCastDescriptorsList.p_descriptors) && (value instanceof ChemCastDescriptorsList)) 
            setDescriptors((ChemCastDescriptorsList)value);
        else
            if (parameter.equals("Fingerprints")) 
                try {
                    convertor.setFingerprints(Boolean.parseBoolean(value.toString()));
                    setPredicting(false);
                } catch (Exception x) {
                    convertor.setFingerprints(false);
                }

	}

	public Object getParameter(Object parameter) {
		if ("Fingerprints".equals(parameter)) return new Boolean(convertor.isFingerprints());
		else if (ChemCastDescriptorsList.p_descriptors.equals(parameter))return getDescriptors();
		else if (p_classification_property.equals(parameter)) return getClassificationProperty();
		else if (p_predicted_property.equals(parameter)) return getPredictedProperty();
		else return null;
	}
	public boolean isUseAtomEnvironments() {
		return useAtomEnvironments;
	}

	public void setUseAtomEnvironments(boolean useAtomEnvironments) {
		this.useAtomEnvironments = useAtomEnvironments;
		
	}

	public boolean isUseFingerprints() {
		return convertor.isFingerprints();
	}

	public void setUseFingerprints(boolean useFingerprints) {
		convertor.setFingerprints(useFingerprints);
	}           

	public ChemCastDescriptorsList getDescriptors() {
		return convertor.getDescriptors();
	}
	public void setDescriptors(ChemCastDescriptorsList descriptors) {
		if (descriptors != null)
	        for (int i=0; i < descriptors.getList().size();i++)
	            descriptors.get(i).setEnabled(true);
		convertor.setDescriptors(descriptors);
        setPredicting(false);
	}
    
	public String getClassProperty() {
		return convertor.getClassProperty();
	}
	public void setClassProperty(String classProperty) {
		convertor.setClassProperty(classProperty);
		setPredicting(false);
	}	
    
	public abstract String getStatus(boolean verbose);

	public boolean isInProgress() {
		return inProgress;
	}

	public void setInProgress(boolean inProgress) {
		this.inProgress = inProgress;
	}


	public String getClassificationProperty() {
		return classificationProperty;
	}

	public void setClassificationProperty(String classificationProperty) {
		this.classificationProperty = classificationProperty;
	}

	public String getPredictedProperty() {
		return predictedProperty;
	}

	public void setPredictedProperty(String predictedProperty) {
		this.predictedProperty = predictedProperty;
	}

}
abstract class WekaSimilarityModelEditor extends SimilarityProcessorEditor {
	/**
	 * 
	 */
	private static final long serialVersionUID = -173937342874902993L;
    
    //protected JButton detailsButton;

    protected GenericObjectEditor m_editor = null;
    
	public WekaSimilarityModelEditor(AbstractWekaModel model) {
		super(model);
       
	}
    protected abstract Object getObject();

    protected GenericObjectEditor createGenericEditor() {
        if (m_editor ==null) {
            m_editor =  new GenericObjectEditor();
            Object o = getObject();
            m_editor.setClassType(o.getClass());
            m_editor.setValue(o);
            m_editor.addPropertyChangeListener(new PropertyChangeListener() {
              public void propertyChange(PropertyChangeEvent e) {
                  if (e.getSource() instanceof Component)
                      ((Component)e.getSource()).repaint();
              }
            });
        }
        return m_editor;
    }    
 

    public void showDetails() {
        GenericObjectEditor e = createGenericEditor();
        int x = getLocationOnScreen().x;
        int y = getLocationOnScreen().y;
        
        PropertyDialog m_PD = new PropertyDialog(e, x, y);
        m_PD.setTitle("Details");
        m_PD.setAlwaysOnTop(true);
        m_PD.setVisible(true);
        //return m_editor.getCustomEditor();
    }

}

 

