/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import weka.core.Instance;
import weka.core.Instances;

/**
 * Jaccard coefficient. 
 * <br>Cab=Zab/(Zaa+Zbb-Zab) 
 * or <br>
 * Sst = Sum(Xsj*Xtj)/[Sum(Xsj*Xsj)+Sum(Xtj*Xtj)-Sum(Xsj*Xtj)]
 * <br> Todeschini,Consonni, "Handbook of Molecular Descriptors", p.399
 * 
 * @author Nina Jeliazkova
 *
 */
public class JaccardSimilarity extends AbstractSimilarity {

	/**
	 * 
	 */
	private static final long serialVersionUID = -1796919034119158442L;
	public JaccardSimilarity() {
		super();
		// TODO Auto-generated constructor stub
	}

	public JaccardSimilarity(Instances data) {
		super(data);

	}
    public float getDistance(Instance a, Instance b) throws Exception {
        // TODO Auto-generated method stub
        return (float) distance(a, b);
    }

    public float getNativeComparison(Instance a, Instance b) throws Exception {
        double zab =zAB(a,b);
        double d = zAA(a)+zAA(b)-zab;
        if (d != 0)
        return  (float)(zab/d);
        else return 0;
    }    
	@Override
	public double distance(Instance a, Instance b) throws Exception {
		return distance(a,b,Double.MAX_VALUE,false);
	}
	@Override
	public double distance(Instance a, Instance b, double arg2) {
		return distance(a, b, arg2, false);
	}
	@Override
	public double distance(Instance a, Instance b, double arg2, boolean print) {
		double zab =zAB(a,b);
		double d = zAA(a)+zAA(b)-zab;
		if (d != 0)
		return  similarity2distance(zab/d);
		else return 0;
	}	
	@Override
	/**
	 * Do nothing
	 */
	public void postProcessDistances(double[] distances) {
        /*
	    for(int i=0; i<distances.length; i++) 
	        distances[i] = 1-distances[i];
		*/
	}
    @Override
    public String toString() {
        return "Jaccard coefficient = Sum(Xsj*Xtj)/[Sum(Xsj*Xsj)+Sum(Xtj*Xtj)-Sum(Xsj*Xtj)]";
    }



}


