/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import java.util.BitSet;

import org.openscience.cdk.interfaces.IAtomContainer;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import ambit.data.molecule.IAtomContainersList;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.FingerprintGenerator;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

public class Subsets2Instances {
	protected static String AttributeSubset = "Subset";
	protected static String AttributeIndex = "Item";
	protected ChemCastDescriptorsList descriptors;
	protected String classProperty;
	protected boolean fingerprints=false;
	protected static FingerprintGenerator fpGen = new FingerprintGenerator(1024);
	protected FastVector attributes = null;
	protected InstanceProcessor instanceProcessor = new InstanceAdder();
	protected boolean useClassProperty = true;
	protected boolean emptyClassProperty = false;
	
	public Subsets2Instances() {
		this(null,null,false);
       
	}
	public Subsets2Instances(ChemCastDescriptorsList descriptors, String classProperty, boolean fingerprints) {
		super();
	}
    public FastVector properties2attributes(Subsets subsets) throws Exception {
        FastVector attributes = new FastVector();
        //attributes.addElement(new Attribute("ID"));
        
        if (descriptors != null)
	        for (int i=0; i < descriptors.getList().size(); i++ )
	            if (descriptors.get(i).isEnabled())
	                attributes.addElement(new Attribute(descriptors.get(i).getName()));

        if (fingerprints) {
            for (int i=0; i < fpGen.getFPLength(); i++ )
                attributes.addElement(new FingerprintAttribute("FP"+i,i)); //,fvClassVal,i));
        }
        if (attributes.size() == 1) throw new Exception("No attributes!");

        if (useClassProperty)
	        if (classProperty == null) {
	            FastVector classVal = new FastVector();
	            for (int i=0; i < subsets.size(); i++ )
	                if ((subsets.get(i).isEnabled()) && (subsets.get(i).getName()!=null))
	                    classVal.addElement(subsets.get(i).getName());
	            attributes.addElement(new Attribute("Group", classVal));
	        } else    
	            attributes.addElement(new Attribute(classProperty));

        /*
        TODO error when using Remove filter - Filter didn't make the test instance immediately available!
        These attributes were intended to provide Instance identification , i.e. subset index and item index
        When passing through Remove filter , building classification model from filtered instances gives the error above.
        attributes.addElement(new Attribute(AttributeSubset,(FastVector)null)); //these will be STRING attributes
        attributes.addElement(new Attribute(AttributeIndex,(FastVector)null));        
         * 
         */
        
        return attributes;
    }
    public Instance createInstance() throws Exception {
        if (attributes == null) throw new Exception("Attributed not defined!");
        return new Instance(attributes.size());
    }
    public Instance property2Instance(Subset subset, int index, Instance instance) throws Exception {
    	return property2Instance(subset.getData(),index,instance,subset.getName());
    }
    public Instance property2Instance(IAtomContainersList data, int index, Instance instance, String className) throws Exception {    	
            
            //instance.setValue(0,index);
            for (int a=0; a < attributes.size();a++) {
                
                Attribute att = (Attribute)attributes.elementAt(a);
                try {
                    if (att instanceof FingerprintAttribute) {
                        Object b = data.getProperty(index,AmbitCONSTANTS.Fingerprint);
                        if ((b ==null) || !(b instanceof BitSet)) {
                            Object ac = fpGen.process(data.getAtomContainer(index));
                            b = ((IAtomContainer) ac).getProperty(AmbitCONSTANTS.Fingerprint);
                            data.setProperty(index,AmbitCONSTANTS.Fingerprint,b);
                            //Object bb = data.getProperty(c,AmbitCONSTANTS.Fingerprint);
                            //System.out.println(b);
                        }
                        boolean bit = ((BitSet)b).get(((FingerprintAttribute)att).getFpIndex());
                        if (bit)  instance.setValue(att, 1); else   instance.setValue(att, 0);

                    } else
                        instance.setValue(att, Double.parseDouble(
                                data.getProperty(index, att.name()).toString() ));
                } catch (Exception x) {
                    //System.err.println(att.name() + "\t"+ x.getMessage());
                    instance.setValue(att,Double.NaN);
                }
            }
            if (useClassProperty && (classProperty == null)) 
             try {
                instance.setClassValue(className);
            } catch (Exception x) {
                //instance.setClassValue(null);
            }
            return instance;
    }
    public Instance property2Instance(IAtomContainer atomContainer, Instance instance, String className) throws Exception {    	
        
        //instance.setValue(0,index);
        for (int a=0; a < attributes.size();a++) {
            
            Attribute att = (Attribute)attributes.elementAt(a);
            try {
                if (att instanceof FingerprintAttribute) {
                    Object b = atomContainer.getProperty(AmbitCONSTANTS.Fingerprint);
                    if ((b ==null) || !(b instanceof BitSet)) {
                        Object ac = fpGen.process(atomContainer);
                        b = ((IAtomContainer) ac).getProperty(AmbitCONSTANTS.Fingerprint);
                        atomContainer.setProperty(AmbitCONSTANTS.Fingerprint,b);
                        //Object bb = data.getProperty(c,AmbitCONSTANTS.Fingerprint);
                        //System.out.println(b);
                    }
                    boolean bit = ((BitSet)b).get(((FingerprintAttribute)att).getFpIndex());
                    if (bit)  instance.setValue(att, 1); else   instance.setValue(att, 0);

                } else
                    instance.setValue(att, Double.parseDouble(
                    		atomContainer.getProperty(att.name()).toString() ));
            } catch (Exception x) {
                //System.err.println(att.name() + "\t"+ x.getMessage());
                instance.setValue(att,Double.NaN);
            }
        }
        if (useClassProperty && (classProperty == null)) 
         try {
            instance.setClassValue(className);
        } catch (Exception x) {
            //instance.setClassValue(null);
        }
        return instance;
    }
    
    public Instances createInstances(String name) throws Exception {
        if (attributes == null) throw new Exception("Attributes not defined!");
        return new Instances(name,attributes,2);
    }
    public Instances properties2Instances(Subsets subsets) throws Exception {
        if (attributes == null) attributes= properties2attributes(subsets);
        Instance instance = createInstance();
        
        Instances instances = createInstances(subsets.toString());
        if (useClassProperty)
        	instances.setClassIndex(attributes.size()-1);
        instance.setDataset(instances);
        for (int i=0; i < subsets.size(); i++ )
            if (subsets.get(i).isEnabled()) 
                try {
                    Subset subset = subsets.get(i);
                    IAtomContainersList data = subset.getData();
                    for (int c=0; c < data.getAtomContainerCount();c++) {
                        property2Instance(subset, c, instance);

                        try {
                        	instanceProcessor.processInstance(subsets,i,c,instances, instance);
                        } catch (Exception x) {
                        	x.printStackTrace();
                        }
                        
                    }
                } catch (Exception x) {
                    x.printStackTrace();
                }
          return instances;      
                
    }
	public boolean isFingerprints() {
		return fingerprints;
	}
	public void setFingerprints(boolean fingerprints) {
		this.fingerprints = fingerprints;
		attributes = null;
	}
	public String getClassProperty() {
		return classProperty;
	}
    
	public void setClassProperty(String classProperty) {
		if ("".equals(classProperty)) { 
			this.classProperty = null;
			setEmptyClassProperty(true);
		} else { 
			this.classProperty = classProperty;
			setEmptyClassProperty(false);
		}	
		attributes = null;
	}
    
	public ChemCastDescriptorsList getDescriptors() {
		return descriptors;
	}
	public void setDescriptors(ChemCastDescriptorsList descriptors) {
		this.descriptors = descriptors;
		attributes = null;
	}
	public InstanceProcessor getInstanceProcessor() {
		return instanceProcessor;
	}
	public void setInstanceProcessor(InstanceProcessor instanceProcessor) {
		this.instanceProcessor = instanceProcessor;
	}
    public synchronized FastVector getAttributes() {
        return attributes;
    }
    public synchronized void setAttributes(FastVector attributes) {
        this.attributes = attributes;
    }
    
	public boolean isUseClassProperty() {
		return useClassProperty;
	}
    
	public void setUseClassProperty(boolean useClassProperty) {
		this.useClassProperty = useClassProperty;
	}
    
    
	public boolean isEmptyClassProperty() {
		return emptyClassProperty;
	}

	public void setEmptyClassProperty(boolean emptyClassProperty) {
		this.emptyClassProperty = emptyClassProperty;
	}
    


}


