/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test.io.batch;

import java.io.File;
import java.util.Observable;
import java.util.Observer;

import junit.framework.TestCase;
import toxTree.exceptions.DecisionMethodException;
import toxTree.io.batch.BatchFactory;
import toxTree.io.batch.BatchProcessing;
import toxTree.io.batch.BatchProcessingException;
import toxTree.io.batch.ToxTreeBatchProcessing;
import toxTree.logging.TTLogger;
import toxTree.tree.cramer.CramerRules;

/**
 * TODO add description
 * @author Nina Jeliazkova
 * <b>Modified</b> 2005-9-4
 */
public class ToxTreeBatchProcessingTest extends TestCase {
	static TTLogger logger = new TTLogger(ToxTreeBatchProcessing.class); 
	protected ToxTreeBatchProcessing batch = null;
	protected String config = "data/Misc/batch.cfg";
	protected String configInterrupted = "data/Misc/batchInterrupted.cfg";
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(ToxTreeBatchProcessingTest.class);
	}

	/*
	 * @see TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
	}

	/*
	 * @see TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Constructor for ToxTreeBatchProcessingTest.
	 * @param arg0
	 */
	public ToxTreeBatchProcessingTest(String arg0) {
		super(arg0);
		try {
			CramerRules rules = null;
			try {
				rules = new CramerRules();
			} catch (DecisionMethodException x) {
				fail();
			}
			batch = new ToxTreeBatchProcessing(
					"data/Misc/test.sdf",
					"data/Misc/batchResults.sdf"
					);
			((ToxTreeBatchProcessing) batch).setDecisionMethod(rules);
			File f = new File(config);
			batch.setConfigFile(f);
			batch.saveConfig();
			batch.close();
			batch = null;

		} catch (BatchProcessingException x) {
			x.printStackTrace();
		}
		TTLogger.configureLog4j(true);
	}
	
	public void testSuccessfullBatch() {
		try {
			ToxTreeBatchProcessing bp = new ToxTreeBatchProcessing(
					"data/Misc/test.sdf",
					"data/Misc/batchResults.sdf"
					);
			bp.setDecisionMethod(new CramerRules());
			bp.addObserver(new Observer() {
				public void update(Observable o, Object arg) {
					BatchProcessing b =	(BatchProcessing) o;
					b.deleteObservers();
					assertTrue(b.getConfigFile().exists());
				}
			});
			bp.start();
			assertFalse(bp.getConfigFile().exists()); //tmp files deleted on success
			assertTrue(bp.isStatus(BatchProcessing.STATUS_FINISHED));
			assertEquals(bp.getReadRecordsCount(),29);
			assertEquals(bp.getWrittenRecordsCount(),29);
		} catch (DecisionMethodException x) {
			fail();
		} catch (BatchProcessingException x) {
			x.printStackTrace();
			fail();
		}
	}
	/*
	public void testBatchInThread() {
		try {
			ToxTreeBatchProcessing bp = new ToxTreeBatchProcessing(
					"cellbox:/src/toxTree/src/toxTree/config/test.sdf",
					"cellbox:/src/toxTree/src/toxTree/config/batchResults.sdf"
					);
			bp.setDecisionMethod(new CramerRules());			
			Thread thread = new Thread(bp);
			thread.run();
			
			assertTrue(bp.isStatus(BatchProcessing.STATUS_FINISHED));
			assertEquals(bp.getReadRecordsCount(),29);
			assertEquals(bp.getWrittenRecordsCount(),29);
			
		} catch (BatchProcessingException x) {
			x.printStackTrace();
			fail();
		}
	}
	*/
	
	public void testBatchInThread() {
		try {
			ToxTreeBatchProcessing bp = new ToxTreeBatchProcessing(
					"data/Misc/test.sdf",
					//"cellbox:/nina/Databases/ligand_info_subset_1_dos.sdf", space after property line
					//"cellbox:/nina/Databases/nciopen_3D_fixed.sdf",
					"data/Misc/batchResults.sdf"
					);
			
			bp.addObserver(new Observer() {

				public void update(Observable o, Object arg) {
					if (o instanceof BatchProcessing) {
						System.out.println(((BatchProcessing) o));

					}

				}
			});
					
			bp.setDecisionMethod(new CramerRules());			
			bp.start();
			
			
			assertTrue(bp.isStatus(BatchProcessing.STATUS_FINISHED));
			assertEquals(bp.getReadRecordsCount(),29);
			assertEquals(bp.getWrittenRecordsCount(),29);
		} catch (DecisionMethodException x) {
			fail();			
		} catch (BatchProcessingException x) {
			x.printStackTrace();
			fail();
		}
	}
	
	public void testBatchFromConfig() {
		try {
			ToxTreeBatchProcessing bp = (ToxTreeBatchProcessing)BatchFactory.createFromConfig(new File(config));
			assertNotNull(bp.getDecisionMethod());
//			bp.setDecisionMethod(new CramerRules());			
			bp.addObserver(new Observer() {

				public void update(Observable o, Object arg) {
					if (o instanceof BatchProcessing) {			
						BatchProcessing b =	(BatchProcessing) o;
						System.out.println(b);
						if (b.getReadRecordsCount()> 11) 
							try {
								b.deleteObservers();
								b.pause();
								System.out.println(b);
							} catch (BatchProcessingException x) {
								x.printStackTrace();
								fail();
							}

					}

				}
			});			
			bp.start();
			
			assertTrue(bp.isStatus(BatchProcessing.STATUS_PAUSED));
			assertEquals(bp.getReadRecordsCount(),21);
			assertEquals(bp.getWrittenRecordsCount(),21);
			
			bp.addObserver(new Observer() {
				/* (non-Javadoc)
				 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
				 */
				public void update(Observable o, Object arg) {
					System.out.println("Continued ...\t" + o);

				}
			}
					);
			bp.start();
			
		} catch (BatchProcessingException x) {
			x.printStackTrace();
			fail();
		}
	}
	
	public void testInterruptedJob() {
		
		try {
			batch = new ToxTreeBatchProcessing(
					"data/Misc/test.sdf",
					"data/Misc/batchInterruptedResults.sdf"
					);
			batch.setDecisionMethod(new CramerRules());
			File f = new File(configInterrupted);
			batch.setConfigFile(f);
			//batch.saveConfig();
			((Observable)batch).addObserver(new Observer() {
				public void update(Observable o, Object arg) {
					if (((BatchProcessing) o).getReadRecordsCount() > 11) {
						((BatchProcessing) o).deleteObservers();
						try {
						((BatchProcessing) o).pause();
						} catch (BatchProcessingException x) {
							x.printStackTrace();
						}
					}

				}
			});
			batch.start();
			assertEquals(batch.getReadRecordsCount(),21);
			System.err.println("???\t" + batch);
			batch.saveConfig();  //configuration should be saved after paused batch
			batch.cancel(); //otherwise files will not be closed !!!!!!!!!!!
			batch.close();
			batch = null;			

//------------------			
			ToxTreeBatchProcessing bp = (ToxTreeBatchProcessing)BatchFactory.
							createFromConfig(new File(configInterrupted));
			
			assertEquals(BatchProcessing.STATUS_PAUSED,bp.getStatus());
			
			assertEquals(21,bp.getReadRecordsCount());
			assertEquals(21,bp.getWrittenRecordsCount());		

			/*
//			bp.setDecisionMethod(new CramerRules());			
			bp.addObserver(new Observer() {

				public void update(Observable o, Object arg) {
					if (o instanceof BatchProcessing) {			
						BatchProcessing b =	(BatchProcessing) o;
						System.out.println(b);
						if (b.getReadRecordsCount()> 11) 
							try {
								b.deleteObservers();
								b.pause();
								System.out.println(b);
							} catch (BatchProcessingException x) {
								x.printStackTrace();
								fail();
							}

					}

				}
			});			
			bp.start();
			
			assertTrue(bp.isStatus(BatchProcessing.STATUS_PAUSED));
			assertEquals(bp.getReadRecordsCount(),21);
			assertEquals(bp.getWrittenRecordsCount(),21);
			
			bp.addObserver(new Observer() {
				public void update(Observable o, Object arg) {
					System.out.println("Continued ...\t" + o);

				}
			}
					);
			bp.start();
*/
		} catch (DecisionMethodException x) {
			fail();			
		} catch (BatchProcessingException x) {
			x.printStackTrace();
			fail();
		}
	}	

}
