/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test.tree.cramer;

import junit.framework.TestCase;

import org.openscience.cdk.Molecule;

import toxTree.core.IDecisionRule;
import toxTree.exceptions.DecisionMethodException;
import toxTree.exceptions.MolAnalyseException;
import toxTree.logging.TTLogger;
import toxTree.query.FunctionalGroups;
import toxTree.query.MolAnalyser;
import toxTree.tree.rules.RuleCarbohydrate;

public class RuleCarbohydrateTest extends TestCase {

	public RuleCarbohydrateTest(String arg0) {
		super(arg0);
		TTLogger.configureLog4j(true);		
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}
	public void testRuleCarbohydrate() {
	    IDecisionRule rule = new RuleCarbohydrate();
	    //
	    Object[][] answer = {
//PG Cramer appendix	    		
	    		{"OCC(O)C",new Boolean(false)},
//	    		BG Cramer appendix	    		
	    		{"OCCC(O)C",new Boolean(false)},
//	    		Actn Cramer appendix	    		
	    		{"O=C(C(O)CC)C",new Boolean(false)},	    			    		
//	    	DHAC, Cramer appendix	    	
	            {"O=C1OC(=CC(=O)C1C(=O)C)C",new Boolean(false)},
//	          Cramer, xylose	            
	            {"OC1COC(O)C(O)C1(O)",new Boolean(true)},
//	          Munro, xylitol
	            {"OCC(O)C(O)C(O)CO",new Boolean(true)},
	            
//	          Munro,  sucrose monopalmitate
	            {"CCCCCCCCCCCCCCCC(=O)O[C@]1(O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)[C@@]2(CO)O[C@H](CO)[C@@H](O)[C@@H]2O",new Boolean(true)}, 
// Munro, sucrose monostearate
	            {"CCCCCCCCCCCCCCCCCC(=O)O[C@]1(O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)[C@@]2(CO)O[C@H](CO)[C@@H](O)[C@@H]2O",new Boolean(true)}
	            
	            };
        
	    for (int i = 0; i < answer.length; i++) {
	        Molecule mol = (Molecule) FunctionalGroups.createAtomContainer((String)answer[i][0]);
	        if (mol != null)
	            try {
	            	System.out.println(answer[i][0]);
	            	MolAnalyser.analyse(mol);
	                Boolean b = (Boolean) answer[i][1];
	                assertEquals(b.booleanValue(),rule.verifyRule(mol));
	            } catch (DecisionMethodException x) {
	                x.printStackTrace();
	            } catch (MolAnalyseException x) {
	                x.printStackTrace();
	            }
	    }
	    
	}
	

}
