/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.ui.tree;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Observable;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import toxTree.core.IDecisionCategory;
import toxTree.core.IDecisionMethod;
import toxTree.core.IDecisionRule;
import toxTree.tree.DecisionNode;
import toxTree.ui.tree.rules.RulePanel;

/**
 * Displays a clickable decision tree. On node click, the details of node and rule are displayed
 * in {@link DecisionNodePanel} and {@link RulePanel}. On right click a popup menu with {@link EditTreeActions} is launched.
 * @author Nina Jeliazkova
 * <b>Modified</b> 2005-10-9
 */
public class TreeDrawing extends JPanel {
	protected RulePanel rulePanel = null;
	protected DecisionNodePanel nodePanel = null;
	protected JCustomTreeComponent treeComponent = null;
	protected EditTreeActions actions ;
	
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = -8122365540682052287L;

	/**
	 * 
	 */
	public TreeDrawing(IDecisionMethod treeMethod, EditTreeActions actions) {
		super(new BorderLayout());
		addWidgets(treeMethod, actions);
	}

	protected void addWidgets(IDecisionMethod treeMethod, EditTreeActions actions) {
		this.actions = actions;
		JPanel p = new JPanel(new BorderLayout());
		treeComponent = new JCustomTreeComponent(treeMethod,null);

		// Listen for when the selection changes.
		treeComponent.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				Object o = treeComponent.objectAt(e.getX(),e.getY());
				if (o != null) {
					if (e.getButton() == MouseEvent.BUTTON1) {
						if (o instanceof IDecisionRule)
							setNode((IDecisionRule) o);
						
					} else {
				        if (e.isPopupTrigger()) {
				        	treeComponent.popup.show(e.getComponent(),
				                       e.getX(), e.getY());
				        }
						
					}
					
				}	
			}
		});

		JScrollPane treeView = new JScrollPane(treeComponent);
		treeView.setPreferredSize(new Dimension(500, 300));

		rulePanel = new RulePanel(null);

		p.add(treeView, BorderLayout.CENTER);
		
    	//ActionMap actionsNode =  null;
    	
    	if (treeMethod.isEditable() && (actions != null)) {
    		/*
    		actionsNode = new ActionMap();
	    	noBranchAction = new SetBranchAction(treeMethod,false,"Modify");
	    	actionsNode.put(DecisionNodePanel.ACTION_MODIFYNOBRANCH,noBranchAction);
	    	yesBranchAction = new SetBranchAction(treeMethod,true,"Modify");
	    	actionsNode.put(DecisionNodePanel.ACTION_MODIFYYESBRANCH,yesBranchAction);
	    	*/
	    	nodePanel = new DecisionNodePanel(actions.getTreeNodeActions());
    	} else nodePanel = new DecisionNodePanel(null);
    	
		nodePanel.setEditable(treeMethod.isEditable());
		p.add(nodePanel, BorderLayout.SOUTH);

		JSplitPane treePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, p,
				rulePanel);
		treePane.setOneTouchExpandable(false);
		treePane.setDividerLocation(250);

		// Provide minimum sizes for the two components in the split pane
		Dimension minimumSize = new Dimension(100, 50);
		p.setMinimumSize(minimumSize);
		rulePanel.setMinimumSize(minimumSize);

		treePane.setBorder(BorderFactory.createTitledBorder("Decision Tree"));

		add(treePane, BorderLayout.CENTER);
	}
	public void setDecisionMethod(IDecisionMethod method) {
		treeComponent.setMethod(method);
	}
	public IDecisionMethod getDecisionMethod() {
		return treeComponent.getMethod();
	}	
	public void setNode(IDecisionRule node) {
		if (node instanceof IDecisionRule) {
			IDecisionRule nodeR = (IDecisionRule) node;
			nodePanel.setNode((Observable) node);
			if (node instanceof DecisionNode) {
				if (actions != null)
					actions.setNode((DecisionNode) node);
			}

			rulePanel.setRule(nodeR);
			IDecisionCategory category;
			IDecisionRule branch = treeComponent.getMethod().getBranch(nodeR,
					false);

			if (branch != null)
				nodePanel.setNoBranch((Observable) branch);
			else {
				category = treeComponent.getMethod().getCategory(nodeR, false);
				if (category == null) {
					// shouldn't get here
					System.err.print("ERROR: no branch and no category!");
					System.err.print(nodeR.getNum());
					System.err.print("\t");
					System.err.println(nodeR);
				}
				nodePanel.setNoBranch((Observable) category);
			}

			branch = treeComponent.getMethod().getBranch(nodeR, true);
			if (branch != null)
				nodePanel.setYesBranch((Observable) branch);

			else {
				category = treeComponent.getMethod().getCategory(nodeR, true);
				nodePanel.setYesBranch((Observable) category);
			}
		}		
	}
}
